/**
 * browser - AOS 유무 반환
 * @memberof module:browser
 * @function
 * @return {Boolean}
 * @example
import { isAOS } from '@nonoll/code-snippet/browser';
console.log(isAOS()); // true or false
 */
const isAOS = () => {
    return /Android/.test(window.navigator.userAgent);
};

/**
 * browser - CHROME 유무 반환
 * @memberof module:browser
 * @function
 * @return {Boolean}
 * @example
import { isCHROME } from '@nonoll/code-snippet/browser';
console.log(isCHROME()); // true or false
 */
const isCHROME = () => {
    return (/chrome|crios/i).test(window.navigator.userAgent);
};

/**
 * browser - IOS 유무 반환
 * @memberof module:browser
 * @function
 * @return {Boolean}
 * @example
import { isIOS } from '@nonoll/code-snippet/browser';
console.log(isIOS()); // true or false
 */
const isIOS = () => {
    return /iP[honeaod;]/.test(window.navigator.userAgent);
};

/**
 * passive 옵션 지원 유무 반환
 * @memberof module:browser
 * @function
 * @return {Boolean}
 * @see https://github.com/WICG/EventListenerOptions/pull/30
 * @see https://github.com/cubiq/iscroll/blob/master/demos/demoUtils.js#L2-L12
 * @see https://developers.google.com/web/tools/lighthouse/audits/passive-event-listeners?hl=ko
 * @example
import { isSupportPassive } from '@nonoll/code-snippet/browser';
console.log(isSupportPassive()); // true or false
 */
/* istanbul ignore next */
const isSupportPassive = () => {
    let supportPassiveOption = false;
    try {
        const options = Object.defineProperty({}, 'passive', { get: () => supportPassiveOption = true });
        addEventListener('test_passive_option', null, options);
    }
    catch (e) { }
    return supportPassiveOption;
};
/**
 * 실행된 browser 에서 지원하는 passive 옵션 값 반환
 * @memberof module:browser
 * @function
 * @return {(IPassive|Boolean)}
 * @see https://developers.google.com/web/tools/lighthouse/audits/passive-event-listeners?hl=ko
 * @example
import { getPassiveOption } from '@nonoll/code-snippet/browser';
console.log(getPassiveOption()); // IPassive or false
 */
const getPassiveOption = () => isSupportPassive() ? { passive: true } : /* istanbul ignore next */ false;

/**
 * @typedef {String} BROWSER_AGENTS
 * @alias BROWSER_AGENTS
 * @memberof module:browser
 * @property {String} AOS - user agent pattern
 * @property {String} IOS - user agent pattern
 */
// for jsdoc
/**
 * @export
 * @readonly
 * @enum {BROWSER_AGENTS}
 */
var BROWSER_AGENTS;
(function (BROWSER_AGENTS) {
    BROWSER_AGENTS["AOS"] = "Android(\\s)([0-9]+)?\\.?([0-9]+)\\.?([0-9]+)?";
    BROWSER_AGENTS["IOS"] = "iP[honeaod;]+(\\sCPU)?\\sOS\\s(\\d+)\\_?(\\d)\\_?(\\d)?";
})(BROWSER_AGENTS || (BROWSER_AGENTS = {}));

/**
 * uaParser
 * @template T, R
 * @memberof module:browser
 * @function
 * @param {T} parsingType parsing 타입
 * @param {Number} [start=1] version slice - start index
 * @param {Number} [end=1] version slice - end index
 * @param {String} [ua=window.navigator.userAgent]
 * @returns {R}
 * @example
import { uaParser, IBrowserType } from '@nonoll/code-snippet/browser';

const iosChromeParser = (): IBrowserType => {
  return uaParser<{CHROME: string}, IBrowserType>({CHROME: 'CriOS\/([0-9]+)?\\.?([0-9]+)\\.?([0-9]+)\\.?([0-9]+)?'}, 1, 6);
};

console.log(iosChromeParser()); // { type, version }
 */
const uaParser = (parsingType, start = 1, end = 4, ua = window.navigator.userAgent) => {
    const matched = Object.entries(parsingType)
        .map((values) => {
        const [type, pattern] = values;
        // tslint:disable-next-line: no-shadowed-variable
        const matched = ua.match(new RegExp(pattern, 'i'));
        if (!matched) {
            return null;
        }
        const version = matched
            .slice(start, end)
            .map(v => v || 0)
            .join('.');
        return { type, version };
    })
        .filter(v => v)[0];
    if (matched) {
        return Object.assign({}, matched);
    }
    else {
        return { type: null, version: '' };
    }
};
/**
 * AOS, IOS - type, version 정보 반환
 * @memberof module:browser
 * @function
 * @returns {IBrowserType}
 * @example
import { browserType } from '@nonoll/code-snippet/browser';

console.log(browserType()); // { type, version }
 */
const browserType = () => uaParser(BROWSER_AGENTS, 2, 5);

/**
 * browser - vendorPrefix 반환
 * @memberof module:browser
 * @see https://davidwalsh.name/vendor-prefix
 * @function
 * @return {IVendorPrefix}
 * @example
import { vendorPrefix } from '@nonoll/code-snippet/browser';
console.log(vendorPrefix()); // {dom: "Webkit", lowercase: "webkit", css: "-webkit-", js: "Webkit"}
 */
const vendorPrefix = () => {
    const styles = window.getComputedStyle(window.document.documentElement, '');
    // tslint:disable-next-line: max-line-length
    const pre = (Array.from(styles).join('').match(/-(moz|webkit|ms)-/) || /* istanbul ignore next */ (styles.OLink === '' && ['', 'o']))[1];
    const dom = ('Webkit|Moz|MS|O').match(new RegExp(`(${pre})`, 'i'))[1];
    return {
        dom,
        lowercase: pre,
        css: `-${pre}-`,
        js: pre[0].toUpperCase() + pre.substr(1)
    };
};

/**
 * @module module:browser
 */

/*
 * Public API Surface of @nonoll/code-snippet/browser
 */

/**
 * Generated bundle index. Do not edit.
 */

export { browserType, getPassiveOption, isAOS, isCHROME, isIOS, isSupportPassive, uaParser, vendorPrefix };
//# sourceMappingURL=nonoll-code-snippet-browser.js.map
