/**
 * debounce
  <iframe
    src="https://codesandbox.io/embed/nonollcode-snippet-9gko8?autoresize=1&expanddevtools=1&fontsize=14&hidenavigation=1&initialpath=%2Ffunctions-debounce.html&module=%2Ffunctions-debounce.html&theme=dark"
    style="width:100%; height:500px; border:1px solid black; border-radius: 4px; overflow:hidden;"
    title="@nonoll/code-snippet"
    allow="geolocation; microphone; camera; midi; vr; accelerometer; gyroscope; payment; ambient-light-sensor; encrypted-media; usb"
    sandbox="allow-modals allow-forms allow-popups allow-scripts allow-same-origin"
  ></iframe>
 * @memberof module:functions
 * @function
 * @see https://css-tricks.com/debouncing-throttling-explained-examples/
 * @param {TypeVoidFunction} fn
 * @param {Number} [delay=0]
 * @returns {TypeVoidFunction}
 * @example
import { debounce } from '@nonoll/code-snippet/functions';

const forExample = () => {
  const delay = 1000 * 1;
  const listener = debounce(res => console.log(res), delay);

  listener('1');
  listener('2');
  listener('3');
  listener(`4 ${new Date()}`);

  setTimeout(() => {
    listener(`5 ${new Date()}`);
  }, delay);
}

forExample();
 */
const debounce = (fn, /* istanbul ignore next */ delay = 0) => {
    let timer;
    return (...args) => {
        window.clearTimeout(timer);
        timer = window.setTimeout(() => {
            fn.apply(null, args);
        }, delay);
    };
};

/**
 * deepFreeze
 * @param {any} target
 * @memberof module:functions
 * @function
 */
const deepFreeze = (target) => {
    if (Object.isFrozen(target)) {
        return target;
    }
    if (!Array.isArray(target) && Object.getPrototypeOf(target) !== Object.getPrototypeOf({})) {
        if (Object.getPrototypeOf(target) !== Object.getPrototypeOf(Function)) {
            throw new Error('not allow types');
        }
    }
    Object.freeze(target);
    Object.keys(target).forEach(key => deepFreeze(target[key]));
    return target;
};

/**
 * empty function
 * @memberof module:functions
 * @alias noop
 */
function noop() { }

/**
 * throttle
  <iframe
    src="https://codesandbox.io/embed/nonollcode-snippet-9gko8?autoresize=1&expanddevtools=1&fontsize=14&hidenavigation=1&initialpath=%2Ffunctions-throttle.html&module=%2Ffunctions-throttle.html&theme=dark"
    style="width:100%; height:500px; border:1px solid black; border-radius: 4px; overflow:hidden;"
    title="@nonoll/code-snippet"
    allow="geolocation; microphone; camera; midi; vr; accelerometer; gyroscope; payment; ambient-light-sensor; encrypted-media; usb"
    sandbox="allow-modals allow-forms allow-popups allow-scripts allow-same-origin"
  ></iframe>
 * @memberof module:functions
 * @function
 * @see https://css-tricks.com/debouncing-throttling-explained-examples/
 * @param {TypeVoidFunction} fn
 * @param {Number} [delay=0]
 * @returns {TypeVoidFunction}
 * @example
import { debounce } from '@nonoll/code-snippet/functions';

const forExample = () => {
  const delay = 1000 * 1;
  const listener = throttle(res => console.log(res), delay);

  listener(`1 ${new Date()}`);
  listener('2');
  listener('3');
  listener('4');

  setTimeout(() => {
    listener(`5 ${new Date()}`);
  }, delay);
}

forExample();
 */
const throttle = (fn, /* istanbul ignore next */ delay = 0) => {
    const caller = (args) => {
        fn.apply(null, args);
        base = +new Date();
    };
    let base = null;
    let isLeading = true;
    let stamp = null;
    return (...args) => {
        if (isLeading) {
            caller(args);
            isLeading = false;
            return;
        }
        stamp = +new Date();
        /* istanbul ignore next */
        base = base || stamp;
        if ((stamp - base) >= delay) {
            /* istanbul ignore next */
            caller(args);
        }
    };
};

/**
 * @module module:functions
 */

/**
 * EventEmitter 의 기능과 동일한 맥락으로, 간단히 구성
  <iframe
    src="https://codesandbox.io/embed/nonollcode-snippet-9gko8?autoresize=1&expanddevtools=1&fontsize=14&hidenavigation=1&initialpath=%2Fevent-EventEmitter.html&module=%2Fevent-EventEmitter.html&theme=dark"
    style="width:100%; height:500px; border:1px solid black; border-radius: 4px; overflow:hidden;"
    title="@nonoll/code-snippet"
    allow="geolocation; microphone; camera; midi; vr; accelerometer; gyroscope; payment; ambient-light-sensor; encrypted-media; usb"
    sandbox="allow-modals allow-forms allow-popups allow-scripts allow-same-origin"
  ></iframe>
 * @export
 * @class EventEmitter
 * @alias event/EventEmitter
 * @memberof event
 * @see https://www.npmjs.com/package/events
 * @example
import { EventEmitter } from '@nonoll/code-snippet/event';

const createElement = ({ tag = 'div', id = '', style = '', value = '', text = '' }) => {
  const doc = window.document;
  const target = doc.createElement(tag);
  target.setAttribute('id', id);
  target.setAttribute('style', style);
  target.setAttribute('value', value);
  if (text) {
    target.textContent = text;
  }
  return target;
}

let ee;
const forExample = () => {
  const doc = window.document;

  const onButton = createElement({ tag: 'button', text: 'on button' });
  const offButton = createElement({ tag: 'button', text: 'off button' });
  const emitButton = createElement({ tag: 'button', text: 'emit button' });
  const EVENT_NAME = 'EVENT_EXAMPLE';

  doc.body.appendChild(onButton);
  doc.body.appendChild(offButton);
  doc.body.appendChild(emitButton);

  ee = new EventEmitter();
  ee.on(EVENT_NAME, res => console.log(res));

  onButton.addEventListener('click', e => {
    e.preventDefault();
    console.log('onButton clicked');
    if (!ee) {
      return;
    }
    ee.off(EVENT_NAME);
    ee.on(EVENT_NAME, res => console.log(res));
  });

  offButton.addEventListener('click', e => {
    e.preventDefault();
    console.log('offButton clicked');
    if (!ee) {
      return;
    }
    ee.off(EVENT_NAME);
  });

  emitButton.addEventListener('click', e => {
    e.preventDefault();
    console.log('emitButton clicked');
    if (!ee) {
      return;
    }
    ee.emit(EVENT_NAME, new Date());
  });
}

forExample();
 */
class EventEmitter {
    constructor() {
        this.events = [];
    }
    /**
     * 이벤트 감지 등록
     * @param {string} eventName
     * @param {TypeVoidFunction} [listener={@link noop}]
     * @param {*} [context]
     * @returns {EventEmitter}
     * @memberof EventEmitter
     */
    on(eventName, /* istanbul ignore next: for noop */ listener = noop, context) {
        this.events.push({ eventName, listener, context });
        return this;
    }
    /**
     * 이벤트 감지 해제
     * @param {string} eventName
     * @param {TypeVoidFunction} [listener]
     * @returns {EventEmitter}
     * @memberof EventEmitter
     */
    off(eventName, listener) {
        // tslint:disable-next-line: max-line-length
        const matched = this.events.findIndex(regEvent => regEvent.eventName === eventName && (listener ? regEvent.listener === listener : true));
        // tslint:disable-next-line: no-bitwise
        if (!~matched) {
            return false;
        }
        this.events.splice(matched, 1);
        return this;
    }
    /**
     * 이벤트 1회 감지 등록
     * @param {string} eventName
     * @param {TypeVoidFunction} [listener={@link noop}]
     * @param {*} [context]
     * @returns {EventEmitter}
     * @memberof EventEmitter
     */
    once(eventName, /* istanbul ignore next: for noop */ listener = noop, context) {
        const onceWrapper = (...values) => {
            this.off(eventName, onceWrapper);
            listener.apply(context || listener, values);
        };
        this.on(eventName, onceWrapper, context);
        return this;
    }
    /**
     * 이벤트 전파
     * @param {string} eventName
     * @param {...any[]} values
     * @returns {EventEmitter}
     * @memberof EventEmitter
     */
    emit(eventName, ...values) {
        this.events.forEach(regEvent => {
            if (regEvent.eventName === eventName) {
                regEvent.listener.apply(regEvent.context || regEvent.listener, values);
            }
        });
        return this;
    }
    /**
     * 이벤트 전파
     * @param {string} eventName
     * @param {any} values
     * @returns {EventEmitter}
     * @memberof EventEmitter
     */
    fire(eventName, values) {
        this.emit(eventName, values);
        return this;
    }
}

;

/**
 * @alias event
 */

/*
 * Public API Surface of @nonoll/code-snippet/event
 */

/**
 * Generated bundle index. Do not edit.
 */

export { EventEmitter };
//# sourceMappingURL=nonoll-code-snippet-event.js.map
