/**
 * browser - AOS 유무 반환
 * @memberof module:browser
 * @function
 * @return {Boolean}
 * @example
import { isAOS } from '@nonoll/code-snippet/browser';
console.log(isAOS()); // true or false
 */
const isAOS = () => {
    return /Android/.test(window.navigator.userAgent);
};

/**
 * browser - CHROME 유무 반환
 * @memberof module:browser
 * @function
 * @return {Boolean}
 * @example
import { isCHROME } from '@nonoll/code-snippet/browser';
console.log(isCHROME()); // true or false
 */
const isCHROME = () => {
    return (/chrome|crios/i).test(window.navigator.userAgent);
};

/**
 * browser - IOS 유무 반환
 * @memberof module:browser
 * @function
 * @return {Boolean}
 * @example
import { isIOS } from '@nonoll/code-snippet/browser';
console.log(isIOS()); // true or false
 */
const isIOS = () => {
    return /iP[honeaod;]/.test(window.navigator.userAgent);
};

/**
 * passive 옵션 지원 유무 반환
 * @memberof module:browser
 * @function
 * @return {Boolean}
 * @see https://github.com/WICG/EventListenerOptions/pull/30
 * @see https://github.com/cubiq/iscroll/blob/master/demos/demoUtils.js#L2-L12
 * @see https://developers.google.com/web/tools/lighthouse/audits/passive-event-listeners?hl=ko
 * @example
import { isSupportPassive } from '@nonoll/code-snippet/browser';
console.log(isSupportPassive()); // true or false
 */
/* istanbul ignore next */
const isSupportPassive = () => {
    let supportPassiveOption = false;
    try {
        const options = Object.defineProperty({}, 'passive', { get: () => supportPassiveOption = true });
        addEventListener('test_passive_option', null, options);
    }
    catch (e) { }
    return supportPassiveOption;
};
/**
 * 실행된 browser 에서 지원하는 passive 옵션 값 반환
 * @memberof module:browser
 * @function
 * @return {(IPassive|Boolean)}
 * @see https://developers.google.com/web/tools/lighthouse/audits/passive-event-listeners?hl=ko
 * @example
import { getPassiveOption } from '@nonoll/code-snippet/browser';
console.log(getPassiveOption()); // IPassive or false
 */
const getPassiveOption = () => isSupportPassive() ? { passive: true } : /* istanbul ignore next */ false;

/**
 * @typedef {String} BROWSER_AGENTS
 * @alias BROWSER_AGENTS
 * @memberof module:browser
 * @property {String} AOS - user agent pattern
 * @property {String} IOS - user agent pattern
 */
// for jsdoc
/**
 * @export
 * @readonly
 * @enum {BROWSER_AGENTS}
 */
var BROWSER_AGENTS;
(function (BROWSER_AGENTS) {
    BROWSER_AGENTS["AOS"] = "Android(\\s)([0-9]+)?\\.?([0-9]+)\\.?([0-9]+)?";
    BROWSER_AGENTS["IOS"] = "iP[honeaod;]+(\\sCPU)?\\sOS\\s(\\d+)\\_?(\\d)\\_?(\\d)?";
})(BROWSER_AGENTS || (BROWSER_AGENTS = {}));

/**
 * uaParser
 * @template T, R
 * @memberof module:browser
 * @function
 * @param {T} parsingType parsing 타입
 * @param {Number} [start=1] version slice - start index
 * @param {Number} [end=1] version slice - end index
 * @param {String} [ua=window.navigator.userAgent]
 * @returns {R}
 * @example
import { uaParser, IBrowserType } from '@nonoll/code-snippet/browser';

const iosChromeParser = (): IBrowserType => {
  return uaParser<{CHROME: string}, IBrowserType>({CHROME: 'CriOS\/([0-9]+)?\\.?([0-9]+)\\.?([0-9]+)\\.?([0-9]+)?'}, 1, 6);
};

console.log(iosChromeParser()); // { type, version }
 */
const uaParser = (parsingType, start = 1, end = 4, ua = window.navigator.userAgent) => {
    const matched = Object.entries(parsingType)
        .map((values) => {
        const [type, pattern] = values;
        // tslint:disable-next-line: no-shadowed-variable
        const matched = ua.match(new RegExp(pattern, 'i'));
        if (!matched) {
            return null;
        }
        const version = matched
            .slice(start, end)
            .map(v => v || 0)
            .join('.');
        return { type, version };
    })
        .filter(v => v)[0];
    if (matched) {
        return Object.assign({}, matched);
    }
    else {
        return { type: null, version: '' };
    }
};
/**
 * AOS, IOS - type, version 정보 반환
 * @memberof module:browser
 * @function
 * @returns {IBrowserType}
 * @example
import { browserType } from '@nonoll/code-snippet/browser';

console.log(browserType()); // { type, version }
 */
const browserType = () => uaParser(BROWSER_AGENTS, 2, 5);

/**
 * browser - vendorPrefix 반환
 * @memberof module:browser
 * @see https://davidwalsh.name/vendor-prefix
 * @function
 * @return {IVendorPrefix}
 * @example
import { vendorPrefix } from '@nonoll/code-snippet/browser';
console.log(vendorPrefix()); // {dom: "Webkit", lowercase: "webkit", css: "-webkit-", js: "Webkit"}
 */
const vendorPrefix = () => {
    const styles = window.getComputedStyle(window.document.documentElement, '');
    // tslint:disable-next-line: max-line-length
    const pre = (Array.from(styles).join('').match(/-(moz|webkit|ms)-/) || /* istanbul ignore next */ (styles.OLink === '' && ['', 'o']))[1];
    const dom = ('Webkit|Moz|MS|O').match(new RegExp(`(${pre})`, 'i'))[1];
    return {
        dom,
        lowercase: pre,
        css: `-${pre}-`,
        js: pre[0].toUpperCase() + pre.substr(1)
    };
};

/**
 * @module module:browser
 */

/*
 * Public API Surface of @nonoll/code-snippet/browser
 */

var publicApi_browser = /*#__PURE__*/Object.freeze({
    __proto__: null,
    isAOS: isAOS,
    isCHROME: isCHROME,
    isIOS: isIOS,
    isSupportPassive: isSupportPassive,
    getPassiveOption: getPassiveOption,
    uaParser: uaParser,
    browserType: browserType,
    vendorPrefix: vendorPrefix
});

/**
 * debounce
  <iframe
    src="https://codesandbox.io/embed/nonollcode-snippet-9gko8?autoresize=1&expanddevtools=1&fontsize=14&hidenavigation=1&initialpath=%2Ffunctions-debounce.html&module=%2Ffunctions-debounce.html&theme=dark"
    style="width:100%; height:500px; border:1px solid black; border-radius: 4px; overflow:hidden;"
    title="@nonoll/code-snippet"
    allow="geolocation; microphone; camera; midi; vr; accelerometer; gyroscope; payment; ambient-light-sensor; encrypted-media; usb"
    sandbox="allow-modals allow-forms allow-popups allow-scripts allow-same-origin"
  ></iframe>
 * @memberof module:functions
 * @function
 * @see https://css-tricks.com/debouncing-throttling-explained-examples/
 * @param {TypeVoidFunction} fn
 * @param {Number} [delay=0]
 * @returns {TypeVoidFunction}
 * @example
import { debounce } from '@nonoll/code-snippet/functions';

const forExample = () => {
  const delay = 1000 * 1;
  const listener = debounce(res => console.log(res), delay);

  listener('1');
  listener('2');
  listener('3');
  listener(`4 ${new Date()}`);

  setTimeout(() => {
    listener(`5 ${new Date()}`);
  }, delay);
}

forExample();
 */
const debounce = (fn, /* istanbul ignore next */ delay = 0) => {
    let timer;
    return (...args) => {
        window.clearTimeout(timer);
        timer = window.setTimeout(() => {
            fn.apply(null, args);
        }, delay);
    };
};

/**
 * deepFreeze
 * @param {any} target
 * @memberof module:functions
 * @function
 */
const deepFreeze = (target) => {
    if (Object.isFrozen(target)) {
        return target;
    }
    if (!Array.isArray(target) && Object.getPrototypeOf(target) !== Object.getPrototypeOf({})) {
        if (Object.getPrototypeOf(target) !== Object.getPrototypeOf(Function)) {
            throw new Error('not allow types');
        }
    }
    Object.freeze(target);
    Object.keys(target).forEach(key => deepFreeze(target[key]));
    return target;
};

/**
 * empty function
 * @memberof module:functions
 * @alias noop
 */
function noop() { }

/**
 * throttle
  <iframe
    src="https://codesandbox.io/embed/nonollcode-snippet-9gko8?autoresize=1&expanddevtools=1&fontsize=14&hidenavigation=1&initialpath=%2Ffunctions-throttle.html&module=%2Ffunctions-throttle.html&theme=dark"
    style="width:100%; height:500px; border:1px solid black; border-radius: 4px; overflow:hidden;"
    title="@nonoll/code-snippet"
    allow="geolocation; microphone; camera; midi; vr; accelerometer; gyroscope; payment; ambient-light-sensor; encrypted-media; usb"
    sandbox="allow-modals allow-forms allow-popups allow-scripts allow-same-origin"
  ></iframe>
 * @memberof module:functions
 * @function
 * @see https://css-tricks.com/debouncing-throttling-explained-examples/
 * @param {TypeVoidFunction} fn
 * @param {Number} [delay=0]
 * @returns {TypeVoidFunction}
 * @example
import { debounce } from '@nonoll/code-snippet/functions';

const forExample = () => {
  const delay = 1000 * 1;
  const listener = throttle(res => console.log(res), delay);

  listener(`1 ${new Date()}`);
  listener('2');
  listener('3');
  listener('4');

  setTimeout(() => {
    listener(`5 ${new Date()}`);
  }, delay);
}

forExample();
 */
const throttle = (fn, /* istanbul ignore next */ delay = 0) => {
    const caller = (args) => {
        fn.apply(null, args);
        base = +new Date();
    };
    let base = null;
    let isLeading = true;
    let stamp = null;
    return (...args) => {
        if (isLeading) {
            caller(args);
            isLeading = false;
            return;
        }
        stamp = +new Date();
        /* istanbul ignore next */
        base = base || stamp;
        if ((stamp - base) >= delay) {
            /* istanbul ignore next */
            caller(args);
        }
    };
};

/**
 * @module module:functions
 */

/**
 * EventEmitter 의 기능과 동일한 맥락으로, 간단히 구성
  <iframe
    src="https://codesandbox.io/embed/nonollcode-snippet-9gko8?autoresize=1&expanddevtools=1&fontsize=14&hidenavigation=1&initialpath=%2Fevent-EventEmitter.html&module=%2Fevent-EventEmitter.html&theme=dark"
    style="width:100%; height:500px; border:1px solid black; border-radius: 4px; overflow:hidden;"
    title="@nonoll/code-snippet"
    allow="geolocation; microphone; camera; midi; vr; accelerometer; gyroscope; payment; ambient-light-sensor; encrypted-media; usb"
    sandbox="allow-modals allow-forms allow-popups allow-scripts allow-same-origin"
  ></iframe>
 * @export
 * @class EventEmitter
 * @alias event/EventEmitter
 * @memberof event
 * @see https://www.npmjs.com/package/events
 * @example
import { EventEmitter } from '@nonoll/code-snippet/event';

const createElement = ({ tag = 'div', id = '', style = '', value = '', text = '' }) => {
  const doc = window.document;
  const target = doc.createElement(tag);
  target.setAttribute('id', id);
  target.setAttribute('style', style);
  target.setAttribute('value', value);
  if (text) {
    target.textContent = text;
  }
  return target;
}

let ee;
const forExample = () => {
  const doc = window.document;

  const onButton = createElement({ tag: 'button', text: 'on button' });
  const offButton = createElement({ tag: 'button', text: 'off button' });
  const emitButton = createElement({ tag: 'button', text: 'emit button' });
  const EVENT_NAME = 'EVENT_EXAMPLE';

  doc.body.appendChild(onButton);
  doc.body.appendChild(offButton);
  doc.body.appendChild(emitButton);

  ee = new EventEmitter();
  ee.on(EVENT_NAME, res => console.log(res));

  onButton.addEventListener('click', e => {
    e.preventDefault();
    console.log('onButton clicked');
    if (!ee) {
      return;
    }
    ee.off(EVENT_NAME);
    ee.on(EVENT_NAME, res => console.log(res));
  });

  offButton.addEventListener('click', e => {
    e.preventDefault();
    console.log('offButton clicked');
    if (!ee) {
      return;
    }
    ee.off(EVENT_NAME);
  });

  emitButton.addEventListener('click', e => {
    e.preventDefault();
    console.log('emitButton clicked');
    if (!ee) {
      return;
    }
    ee.emit(EVENT_NAME, new Date());
  });
}

forExample();
 */
class EventEmitter {
    constructor() {
        this.events = [];
    }
    /**
     * 이벤트 감지 등록
     * @param {string} eventName
     * @param {TypeVoidFunction} [listener={@link noop}]
     * @param {*} [context]
     * @returns {EventEmitter}
     * @memberof EventEmitter
     */
    on(eventName, /* istanbul ignore next: for noop */ listener = noop, context) {
        this.events.push({ eventName, listener, context });
        return this;
    }
    /**
     * 이벤트 감지 해제
     * @param {string} eventName
     * @param {TypeVoidFunction} [listener]
     * @returns {EventEmitter}
     * @memberof EventEmitter
     */
    off(eventName, listener) {
        // tslint:disable-next-line: max-line-length
        const matched = this.events.findIndex(regEvent => regEvent.eventName === eventName && (listener ? regEvent.listener === listener : true));
        // tslint:disable-next-line: no-bitwise
        if (!~matched) {
            return false;
        }
        this.events.splice(matched, 1);
        return this;
    }
    /**
     * 이벤트 1회 감지 등록
     * @param {string} eventName
     * @param {TypeVoidFunction} [listener={@link noop}]
     * @param {*} [context]
     * @returns {EventEmitter}
     * @memberof EventEmitter
     */
    once(eventName, /* istanbul ignore next: for noop */ listener = noop, context) {
        const onceWrapper = (...values) => {
            this.off(eventName, onceWrapper);
            listener.apply(context || listener, values);
        };
        this.on(eventName, onceWrapper, context);
        return this;
    }
    /**
     * 이벤트 전파
     * @param {string} eventName
     * @param {...any[]} values
     * @returns {EventEmitter}
     * @memberof EventEmitter
     */
    emit(eventName, ...values) {
        this.events.forEach(regEvent => {
            if (regEvent.eventName === eventName) {
                regEvent.listener.apply(regEvent.context || regEvent.listener, values);
            }
        });
        return this;
    }
    /**
     * 이벤트 전파
     * @param {string} eventName
     * @param {any} values
     * @returns {EventEmitter}
     * @memberof EventEmitter
     */
    fire(eventName, values) {
        this.emit(eventName, values);
        return this;
    }
}

;

/**
 * @alias event
 */

/*
 * Public API Surface of @nonoll/code-snippet/event
 */

var publicApi_event = /*#__PURE__*/Object.freeze({
    __proto__: null,
    EventEmitter: EventEmitter
});

/*
 * Public API Surface of @nonoll/code-snippet/functions
 */

var publicApi_functions = /*#__PURE__*/Object.freeze({
    __proto__: null,
    debounce: debounce,
    deepFreeze: deepFreeze,
    noop: noop,
    throttle: throttle
});

/**
 * 최대 공약수(greatest common divisor) 반환
 * @memberof module:math
 * @function
 * @param {Number} a
 * @param {Number} b
 * @returns {Number}
 */
const gcd = (a, b) => {
    if (!b) {
        return a;
    }
    return gcd(b, a % b);
};

/**
 * 최소 공배수(least common multiple) 반환
 * @memberof module:math
 * @function
 * @param {Number} a
 * @param {Number} b
 * @returns {Number}
 */
const lcm = (a, b) => {
    return a * b / gcd(a, b);
};

/**
 * width : height 로 css padding-top 비율(%) 반환<br/>
 * css padding에 의한 비율 계산이 필요할 경우 사용
 * @memberof module:math
 * @function
 * @param {Number} width
 * @param {Number} height
 * @returns {IRatio}
 */
const sizePercentRatio = ({ width, height }) => {
    const w = Math.round(width);
    const h = Math.round(height);
    const ratio = {
        width: w,
        height: h,
        ratio: ((height / width) * 100).toFixed(2)
    };
    return ratio;
};

/**
 * width : height 비율 반환
 * @memberof module:math
 * @function
 * @param {Number} width
 * @param {Number} height
 * @returns {IRatio}
 */
const sizeRatio = ({ width, height }) => {
    const gcdValue = gcd(width, height);
    const w = width / gcdValue;
    const h = height / gcdValue;
    const ratio = {
        width: w,
        height: h,
        ratio: `${w}x${h}`
    };
    return ratio;
};

;
;

/**
 * @module module:math
 */

/*
 * Public API Surface of @nonoll/code-snippet/math
 */

var publicApi_math = /*#__PURE__*/Object.freeze({
    __proto__: null,
    gcd: gcd,
    lcm: lcm,
    sizePercentRatio: sizePercentRatio,
    sizeRatio: sizeRatio
});

const ɵ0 = (_, v) => v / 100;
const defaultThresholdList = new Array(101).fill(0).map(ɵ0);
/**
 * IntersectionEvent Types
 * @event IntersectionObserver#INTERSECTION_EVENTS
 * @memberof IntersectionObserver
 * @property {String} CHANGE - 변경 시점
 */
// for jsdoc
/**
 * @export
 * @readonly
 * @enum {INTERSECTION_EVENTS}
 */
const INTERSECTION_EVENTS = {
    CHANGE: 'INTERSECTION_OBSERVER-EVENTS-CHANGE'
};
/**
 * @typedef {String} INTERSECTION_DIRECTIONS
 * @alias INTERSECTION_DIRECTIONS
 * @memberof IntersectionObserver
 * @property {String} UP
 * @property {String} DOWN
 * @property {String} UNKNOWN
 */
// for jsdoc
/**
 * @export
 * @readonly
 * @enum {INTERSECTION_DIRECTIONS}
 */
var INTERSECTION_DIRECTIONS;
(function (INTERSECTION_DIRECTIONS) {
    INTERSECTION_DIRECTIONS["UP"] = "INTERSECTION_OBSERVER-DIRECTIONS-UP";
    INTERSECTION_DIRECTIONS["DOWN"] = "INTERSECTION_OBSERVER-DIRECTIONS-DOWN";
    INTERSECTION_DIRECTIONS["UNKNOWN"] = "INTERSECTION_OBSERVER-DIRECTIONS-UNKNOWN";
})(INTERSECTION_DIRECTIONS || (INTERSECTION_DIRECTIONS = {}));
;
class ExtIntersectionObserverEntry {
    constructor(initParams) {
        const { target, visibleRatio = 0, options = {} } = initParams;
        /* istanbul ignore next: for not support error */
        if (!target) {
            throw new Error('IntersectionObserver target 설정은 필수 값 입니다.');
        }
        const defParams = {
            root: null,
            rootMargin: '0px',
            threshold: defaultThresholdList
        };
        this.target = target;
        this.visibleRatio = visibleRatio;
        this.options = Object.assign({}, defParams, options);
    }
    checkForIntersections(target, options) {
        const { root, rootMargin } = options;
        const rootMarginValues = this.parseRootMargin(rootMargin);
        const rootIsInDom = this.rootIsInDom(root);
        const rootRect = rootIsInDom ? this.getRootRect(root, rootMarginValues) : this.getEmptyRect();
        const boundingClientRect = this.getBoundingClientRect(target);
        const rootContainsTarget = this.rootContainsTarget(root, target);
        const intersectionRect = rootIsInDom && rootContainsTarget && this.computeTargetAndRootIntersection(target, rootRect, root);
        const isIntersecting = !!intersectionRect;
        const targetArea = boundingClientRect.width * boundingClientRect.height;
        let intersectionRatio = 0;
        if (targetArea) {
            const rect = intersectionRect || this.getEmptyRect();
            const intersectionArea = rect.width * rect.height;
            intersectionRatio = Number((intersectionArea / targetArea).toFixed(4));
        }
        else {
            intersectionRatio = isIntersecting ? 1 : 0;
        }
        return {
            boundingClientRect,
            intersectionRatio,
            intersectionRect: intersectionRect || this.getEmptyRect(),
            isIntersecting,
            rootBounds: rootRect,
            target,
            time: window.performance && performance.now && performance.now()
        };
    }
    computeTargetAndRootIntersection(target = null, rootRect = null, root = null) {
        if (window.getComputedStyle(target).display === 'none') {
            return false;
        }
        const targetRect = this.getBoundingClientRect(target);
        let intersectionRect = targetRect;
        let parent = this.getParentNode(target);
        let atRoot = false;
        while (!atRoot) {
            let parentRect = null;
            const parentComputedStyle = (parent.nodeType === 1 ? window.getComputedStyle(parent) : {});
            if (parentComputedStyle.display === 'none') {
                return;
            }
            if (parent === root) {
                atRoot = true;
                parentRect = rootRect;
            }
            else {
                if (parent !== document.body &&
                    parent !== document.documentElement &&
                    parentComputedStyle.overflow !== 'visible') {
                    parentRect = this.getBoundingClientRect(parent);
                }
            }
            if (parentRect) {
                intersectionRect = this.computeRectIntersection(parentRect, intersectionRect);
                if (!intersectionRect) {
                    break;
                }
            }
            parent = this.getParentNode(parent);
        }
        return intersectionRect;
    }
    computeRectIntersection(rect1, rect2) {
        const top = Math.max(rect1.top, rect2.top);
        const bottom = Math.min(rect1.bottom, rect2.bottom);
        const left = Math.max(rect1.left, rect2.left);
        const right = Math.min(rect1.right, rect2.right);
        const width = right - left;
        const height = bottom - top;
        const values = { top, bottom, left, right, width, height };
        return (width >= 0 && height >= 0) && values;
    }
    parseRootMargin(rootMargin = '0px') {
        const margins = rootMargin.split(/\s+/).map((margin) => {
            const parts = /^(-?\d*\.?\d+)(px|%)$/.exec(margin);
            if (!parts) {
                throw new Error('rootMargin must be specified in pixels or percent');
            }
            return { value: window.parseFloat(parts[1]), unit: parts[2] };
        });
        margins[1] = margins[1] || margins[0];
        margins[2] = margins[2] || margins[0];
        margins[3] = margins[3] || margins[1];
        return margins;
    }
    rootIsInDom(root) {
        return root || this.containsDeep(window.document, root);
    }
    rootContainsTarget(root, target) {
        return this.containsDeep(root || window.document, target);
    }
    containsDeep(parent, child) {
        let node = child;
        while (node) {
            if (node === parent) {
                return true;
            }
            node = this.getParentNode(node);
        }
        return false;
    }
    getParentNode(node) {
        const parent = node.parentNode;
        if (parent && parent.nodeType === 11 && parent.host) {
            // If the parent is a shadow root, return the host element.
            return parent.host;
        }
        return parent;
    }
    getRootRect(root, rootMargin = []) {
        let rootRect;
        if (root) {
            rootRect = this.getBoundingClientRect(root);
        }
        else {
            // Use <html>/<body> instead of window since scroll bars affect size.
            const html = window.document.documentElement;
            const body = window.document.body;
            rootRect = {
                top: 0,
                left: 0,
                right: html.clientWidth || body.clientWidth,
                width: html.clientWidth || body.clientWidth,
                bottom: html.clientHeight || body.clientHeight,
                height: html.clientHeight || body.clientHeight
            };
        }
        return this.expandRectByRootMargin(rootRect, rootMargin);
    }
    getBoundingClientRect(el) {
        let rect = null;
        try {
            rect = el.getBoundingClientRect();
        }
        catch (err) {
            // Ignore Windows 7 IE11 "Unspecified error"
            // https://github.com/w3c/IntersectionObserver/pull/205
        }
        if (!rect) {
            return this.getEmptyRect();
        }
        return rect;
    }
    getEmptyRect() {
        const emptyRect = {
            bottom: 0,
            height: 0,
            left: 0,
            right: 0,
            top: 0,
            width: 0,
            x: 0,
            y: 0
        };
        return Object.assign(Object.assign({}, emptyRect), { toJSON: () => emptyRect });
    }
    expandRectByRootMargin(rect, rootMargin = []) {
        const [mt, mr, mb, ml] = rootMargin.map((margin, i) => {
            return margin.unit === 'px' ? margin.value : (margin.value * (i % 2 ? rect.width : rect.height)) / 100;
        });
        const { top, right, bottom, left } = rect;
        const newRect = {
            top: top - mt,
            right: right + mr,
            bottom: bottom + mb,
            left: left - ml,
            x: 0,
            y: 0,
            width: 0,
            height: 0
        };
        newRect.x = newRect.left;
        newRect.y = newRect.top;
        newRect.width = newRect.right - newRect.left;
        newRect.height = newRect.bottom - newRect.top;
        return Object.assign(Object.assign({}, newRect), { toJSON: () => newRect });
    }
    getChangeData() {
        const entry = this.checkForIntersections(this.target, this.options);
        const intersectionRatio = Math.floor(entry.intersectionRatio * 100);
        const isVisible = entry.intersectionRatio > this.visibleRatio;
        return {
            currentTarget: this.target,
            entry,
            intersectionRatio,
            isVisible,
            direction: INTERSECTION_DIRECTIONS.UNKNOWN
        };
    }
}
class PureIntersectionObserver {
    constructor(callback, options) {
        // TODO polyfill
        // https://www.npmjs.com/package/intersection-observer
        // const IntersectionObserver = require('intersection-observer');
        // this.pure = new IntersectionObserver(callback);
        this.pure = new window.IntersectionObserver(callback, options);
    }
    observe(target, options) {
        this.pure.observe(target, options);
    }
    disconnect() {
        this.pure.disconnect();
    }
}
/**
 * element 가 viewport 영역에 노출되는지 변화를 감지
 * @export
 * @class IntersectionObserver
 * @alias observer/IntersectionObserver
 * @extends {EventEmitter}
 * @see https://developer.mozilla.org/ko/docs/Web/API/IntersectionObserver/IntersectionObserver
 * @see https://caniuse.com/#search=IntersectionObserver
 * @throws {Error} target 옵션을 지정하지 않은 경우 - throw new Error('IntersectionObserver target 설정은 필수 값 입니다.');
 * @param {Partial<IIntersectionObserverExtOption>} initParams
 * @param {Element} initParams.target
 * @param {Number} [initParams.visibleRatio=0]
 * @param {Number} [initParams.throttle=300]
 * @param {Function|null} [initParams.callback={@link noop}] callback function
 * @param {IntersectionObserverInit} [initParams.options={}]
 * @example
 .. TODO
 */
class IntersectionObserver extends EventEmitter {
    constructor(initParams) {
        super();
        this.onIntersectionCallback = (changeData) => { };
        const { target, visibleRatio = 0, throttle: throttle$1 = 300, callback = noop, options = {} } = initParams;
        /* istanbul ignore next: for not support error */
        if (!target) {
            throw new Error('IntersectionObserver target 설정은 필수 값 입니다.');
        }
        const defParams = {
            root: null,
            rootMargin: '0px',
            threshold: defaultThresholdList
        };
        this.target = target;
        this.visibleRatio = visibleRatio;
        this.prevRatio = 0;
        this.onIntersectionCallback = callback;
        this.onIntersectionListener = this.onIntersectionListener.bind(this);
        this.options = Object.assign({}, defParams, options);
        if (throttle$1) {
            this.observer = new PureIntersectionObserver(throttle(this.onIntersectionListener, throttle$1), this.options);
        }
        else {
            this.observer = new PureIntersectionObserver(this.onIntersectionListener, this.options);
        }
    }
    /**
     * element 가 이미 노출된 경우 등, IntersectionObserver 등록으로 판단이 어려울 경우<br>
     * IntersectionObserver.getIntersectionEntry 를 이용하여 target 의 노출 여부를 판단
     * @static
     * @param {Partial<IIntersectionObserverExtOption>} initParams
     * @param {Element} initParams.target
     * @param {Number} [initParams.visibleRatio=0]
     * @param {Number} [initParams.throttle=300]
     * @param {Function|null} [initParams.callback={@link noop}] callback function
     * @param {IntersectionObserverInit} [initParams.options={}]
     * @returns {IIntersectionChangeData}
     * @memberof IntersectionObserver
     * @example
     .. TODO
     */
    static getIntersectionEntry(initParams) {
        return new ExtIntersectionObserverEntry(initParams).getChangeData();
    }
    /**
     * 변화 감지 이벤트 리스너
     * @private
     * @param {IntersectionObserverEntry[]} entries
     * [MDN IntersectionObserverEntry]{@link https://developer.mozilla.org/en-US/docs/Web/API/IntersectionObserverEntry}
     * @memberof IntersectionObserver
     * @fires IntersectionObserver#INTERSECTION_EVENTS
     */
    onIntersectionListener(entries) {
        const entry = entries[0];
        const { intersectionRatio } = entry;
        const isVisible = intersectionRatio > this.visibleRatio;
        const direction = this.prevRatio > intersectionRatio ? INTERSECTION_DIRECTIONS.DOWN : INTERSECTION_DIRECTIONS.UP;
        const changeData = {
            currentTarget: this.target,
            entry,
            intersectionRatio: Math.floor(intersectionRatio * 100),
            isVisible,
            direction
        };
        this.emit(INTERSECTION_EVENTS.CHANGE, changeData);
        this.onIntersectionCallback(changeData);
        this.prevRatio = intersectionRatio;
    }
    /**
     * 이벤트 감지 설정
     * @returns {IntersectionObserver}
     * @memberof IntersectionObserver
     */
    attach() {
        this.observer.observe(this.target, this.options);
        return this;
    }
    /**
     * 이벤트 감지 해제
     * @returns {IntersectionObserver}
     * @memberof IntersectionObserver
     */
    detach() {
        this.observer.disconnect();
        return this;
    }
    /**
     * destroy
     * @returns {IntersectionObserver}
     * @memberof IntersectionObserver
     */
    destroy() {
        this.detach();
        this.observer = null;
        return this;
    }
}

/**
 * @typedef {String} MutationRecordType
 * @alias MutationRecordType
 * @memberof MutationObserver
 * @see https://developer.mozilla.org/ko/docs/Web/API/MutationObserver#MutationObserverInit
 * @property {String} childList
 * @property {String} attributes
 * @property {String} characterData
 * @property {String} subtree
 */
// for jsdoc
/**
 * @export
 * @readonly
 * @enum {MutationRecordType}
 */
var MutationRecordType;
(function (MutationRecordType) {
    MutationRecordType["childList"] = "childList";
    MutationRecordType["attributes"] = "attributes";
    MutationRecordType["characterData"] = "characterData";
    MutationRecordType["subtree"] = "subtree";
})(MutationRecordType || (MutationRecordType = {}));
/**
 * MutationEvent Types
 * @event MutationObserver#MUTATION_EVENTS
 * @memberof MutationObserver
 * @property {String} CHANGE_CHILD_LIST - childList 변경 시점
 * @property {String} CHANGE_SUBTREE - subtree 변경 시점
 * @property {String} CHANGE_ATTRIBUTES - attributes 변경 시점
 * @property {String} CHANGE_CHARACTER_DATA - data 변경 시점
 * @property {String} CHANGE - 기타 변경 시점
 * @property {String} WILD_CARD - 모든 변경 시점
 */
// for jsdoc
/**
 * @export
 * @readonly
 * @enum {MUTATION_EVENTS}
 */
var MUTATION_EVENTS;
(function (MUTATION_EVENTS) {
    MUTATION_EVENTS["CHANGE_CHILD_LIST"] = "MUTATION_OBERSERVER-EVENTS-CHANGE_CHILD_LIST";
    MUTATION_EVENTS["CHANGE_SUBTREE"] = "MUTATION_OBERSERVER-EVENTS-CHANGE_SUBTREE";
    MUTATION_EVENTS["CHANGE_ATTRIBUTES"] = "MUTATION_OBERSERVER-EVENTS-CHANGE_ATTRIBUTES";
    MUTATION_EVENTS["CHANGE_CHARACTER_DATA"] = "MUTATION_OBERSERVER-EVENTS-CHANGE_CHARACTER_DATA";
    MUTATION_EVENTS["CHANGE"] = "MUTATION_OBERSERVER-EVENTS-CHANGE";
    MUTATION_EVENTS["WILD_CARD"] = "MUTATION_OBERSERVER-EVENTS-CHANGE_WILCD_CARD";
})(MUTATION_EVENTS || (MUTATION_EVENTS = {}));
class PureMutationObserver {
    constructor(callback) {
        // TODO polyfill
        // const MutationObserver = require('mutation-observer');
        // this.pure = new MutationObserver(callback);
        this.pure = new window.MutationObserver(callback);
    }
    observe(target, options) {
        this.pure.observe(target, options);
    }
    disconnect() {
        this.pure.disconnect();
    }
}
/**
 * target 으로 설정된 element 의 변화를 감지
  <iframe
    src="https://codesandbox.io/embed/nonollcode-snippet-9gko8?autoresize=1&expanddevtools=1&fontsize=14&hidenavigation=1&initialpath=%2Fobserver-MutationObserver.html&module=%2Fobserver-MutationObserver.html&theme=dark"
    style="width:100%; height:500px; border:1px solid black; border-radius: 4px; overflow:hidden;"
    title="@nonoll/code-snippet"
    allow="geolocation; microphone; camera; midi; vr; accelerometer; gyroscope; payment; ambient-light-sensor; encrypted-media; usb"
    sandbox="allow-modals allow-forms allow-popups allow-scripts allow-same-origin"
  ></iframe>
 * @export
 * @class MutationObserver
 * @alias observer/MutationObserver
 * @extends EventEmitter
 * @see https://developer.mozilla.org/ko/docs/Web/API/MutationObserver
 * @see https://github.com/webmodules/mutation-observer
 * @see https://developer.mozilla.org/ko/docs/Web/API/MutationObserver#MutationObserverInit
 * @param {Partial<IMutationObserverExtOption>} option
 * @param {HTMLElement} [option.target=null] target
 * @param {Number} [option.debounce=300] debounce
 * @param {Function|null} [option.callback={@link noop}] callback function
 * @param {Object} [option.options={}]
 * @param {Boolean} [option.options.childList]
 * @param {Boolean} [option.options.attributes]
 * @param {Boolean} [option.options.characterData]
 * @param {Boolean} [option.options.subtree]
 * @param {Boolean} [option.options.attributeOldValue]
 * @param {Boolean} [option.options.characterDataOldValue]
 * @param {Array.<String>} [option.options.attributeFilter] [MDN attributeFilter]{@link https://developer.mozilla.org/en-US/docs/Web/API/MutationObserverInit/attributeFilter}
 * @example
import { MutationObserver, MUTATION_EVENTS } from '@nonoll/code-snippet/observer';

const createElement = ({ tag = 'div', id = '', style = '', value = '', text = '' }) => {
  const doc = window.document;
  const target = doc.createElement(tag);
  target.setAttribute('id', id);
  target.setAttribute('style', style);
  target.setAttribute('value', value);
  if (text) {
    target.textContent = text;
  }
  return target;
}

let observer;

const forExample = () => {
  if (observer) {
    console.log('already example');
    return;
  }

  const doc = window.document;

  const target = createElement({ id: 'example_target', style: 'border: 1px solid red' });
  doc.body.appendChild(target);

  const options = {
    childList: true,
    subtree: true
  };

  observer = new MutationObserver({ target, options });

  observer.on(MUTATION_EVENTS.WILD_CARD, (type, values) => {
    console.log('wildCard', type, values);
  }).on(MUTATION_EVENTS.CHANGE_CHILD_LIST, values => {
    console.log('childList', values);
  });

  observer.attach();

  const attachButton = createElement({ tag: 'button', text: 'observer attach' });
  const detachButton = createElement({ tag: 'button', text: 'observer detach' });
  const appendButton = createElement({ tag: 'button', text: 'append' });

  doc.body.appendChild(attachButton);
  doc.body.appendChild(detachButton);
  doc.body.appendChild(appendButton);

  attachButton.addEventListener('click', e => {
    e.preventDefault();
    console.log('attachButton clicked');
    if (!observer) {
      return;
    }
    observer.attach();
  });

  detachButton.addEventListener('click', e => {
    e.preventDefault();
    console.log('detachButton clicked');
    if (!observer) {
      return;
    }
    observer.detach();
  });

  appendButton.addEventListener('click', e => {
    e.preventDefault();
    console.log('appendButton clicked');
    if (!observer) {
      return;
    }
    const input = createElement({ tag: 'input', value: `${+new Date()}` });
    target.appendChild(input);
  });
}

forExample();
 */
class MutationObserver extends EventEmitter {
    constructor({ target = null, debounce: debounce$1 = 300, callback = noop, options = {} }) {
        super();
        const defaultOptions = { childList: true, subtree: true };
        this.target = target;
        this.options = Object.assign({}, defaultOptions, options);
        this.onMutationObserverListener = this.onMutationObserverListener.bind(this);
        /* istanbul ignore next: for noop */
        this.callback = callback || noop;
        if (debounce$1) {
            this.observer = new PureMutationObserver(debounce(this.onMutationObserverListener, debounce$1));
        }
        else {
            this.observer = new PureMutationObserver(this.onMutationObserverListener);
        }
    }
    /**
     * 이벤트 감지 설정
     * @returns {MutationObserver}
     * @memberof MutationObserver
     * @example
     observer.attach();
     */
    attach() {
        this.observer.observe(this.target, this.options);
        return this;
    }
    /**
     * 이벤트 감지 등록
     * @override
     * @param {string} eventName
     * @param {TypeVoidFunction} [listener={@link noop}]
     * @param {*} [context]
     * @memberof MutationObserver
     * @listens MutationObserver#MUTATION_EVENTS
     * @returns {MutationObserver}
     */
    on(eventName, /* istanbul ignore next: for noop */ listener = noop, context) {
        super.on(eventName, listener, context);
        return this;
    }
    /**
     * 변화 감지 이벤트 리스너
     * @private
     * @param {MutationRecord[]} mutations [MDN MutationRecord]{@link https://developer.mozilla.org/en-US/docs/Web/API/MutationRecord}
     * @memberof MutationObserver
     * @fires MutationObserver#MUTATION_EVENTS
     */
    onMutationObserverListener(mutations) {
        mutations.forEach(mutation => {
            const type = mutation.type;
            const values = { type, mutation };
            switch (type) {
                case MutationRecordType.childList:
                    this.callback(values);
                    this.emit(MUTATION_EVENTS.CHANGE_CHILD_LIST, values);
                    this.emit(MUTATION_EVENTS.WILD_CARD, MUTATION_EVENTS.CHANGE_CHILD_LIST, values);
                    break;
                case MutationRecordType.subtree:
                    this.callback(values);
                    this.emit(MUTATION_EVENTS.CHANGE_SUBTREE, values);
                    this.emit(MUTATION_EVENTS.WILD_CARD, MUTATION_EVENTS.CHANGE_SUBTREE, values);
                    break;
                case MutationRecordType.attributes:
                    this.callback(values);
                    this.emit(MUTATION_EVENTS.CHANGE_ATTRIBUTES, values);
                    this.emit(MUTATION_EVENTS.WILD_CARD, MUTATION_EVENTS.CHANGE_ATTRIBUTES, values);
                    break;
                case MutationRecordType.characterData:
                    this.callback(values);
                    this.emit(MUTATION_EVENTS.CHANGE_CHARACTER_DATA, values);
                    this.emit(MUTATION_EVENTS.WILD_CARD, MUTATION_EVENTS.CHANGE_CHARACTER_DATA, values);
                    break;
                /* istanbul ignore next: for default */
                default:
                    this.callback(values);
                    this.emit(MUTATION_EVENTS.CHANGE, type, values);
                    this.emit(MUTATION_EVENTS.WILD_CARD, type, values);
                    break;
            }
        });
    }
    /**
     * 이벤트 감지 해제
     * @returns {MutationObserver}
     * @memberof MutationObserver
     * @example
     observer.detach();
     */
    detach() {
        if (this.observer) {
            this.observer.disconnect();
        }
        return this;
    }
    /**
     * destory
     * @returns {MutationObserver}
     * @memberof MutationObserver
     * @example
     observer.destroy();
     */
    destroy() {
        this.detach();
        this.observer = null;
        return this;
    }
}

/**
 * @typedef {String} PREFERS_COLOR_SCHEME
 * @alias PREFERS_COLOR_SCHEME
 * @memberof PrefersColorSchemeObserver
 * @property {String} LIGHT
 * @property {String} DARK
 */
// for jsdoc
/**
 * @export
 * @readonly
 * @enum {PREFERS_COLOR_SCHEME}
 */
var PREFERS_COLOR_SCHEME;
(function (PREFERS_COLOR_SCHEME) {
    PREFERS_COLOR_SCHEME["LIGHT"] = "light";
    PREFERS_COLOR_SCHEME["DARK"] = "dark";
})(PREFERS_COLOR_SCHEME || (PREFERS_COLOR_SCHEME = {}));
/**
 * PrefersColorSchemeObserverEvent Types
 * @event PrefersColorSchemeObserver#PREFERS_COLOR_SCHEME_EVENTS
 * @memberof PrefersColorSchemeObserver
 * @property {String} CHANGE - 변경 시점
 * @property {String} GET_STATUS - 현재 상태값 체크
 */
// for jsdoc
/**
 * @export
 * @readonly
 * @enum {PREFERS_COLOR_SCHEME_EVENTS}
 */
var PREFERS_COLOR_SCHEME_EVENTS;
(function (PREFERS_COLOR_SCHEME_EVENTS) {
    PREFERS_COLOR_SCHEME_EVENTS["CHANGE"] = "PREFERS_COLOR_SCHEME_OBSERVER-EVENTS-CHANGE";
    PREFERS_COLOR_SCHEME_EVENTS["GET_STATUS"] = "PREFERS_COLOR_SCHEME_OBSERVER-EVENTS-GET_COLOR_SCHEME";
})(PREFERS_COLOR_SCHEME_EVENTS || (PREFERS_COLOR_SCHEME_EVENTS = {}));
/**
 * prefers-color-scheme 상태 변화를 감지
<iframe
  src="https://codesandbox.io/embed/nonollcode-snippet-9gko8?autoresize=1&expanddevtools=1&fontsize=14&hidenavigation=1&initialpath=%2Fobsever-PrefersColorScheme.html&module=%2Fobsever-PrefersColorScheme.html&theme=dark"
  style="width:100%; height:500px; border:1px solid black; border-radius: 4px; overflow:hidden;"
  title="@nonoll/code-snippet"
  allow="geolocation; microphone; camera; midi; vr; accelerometer; gyroscope; payment; ambient-light-sensor; encrypted-media; usb"
  sandbox="allow-modals allow-forms allow-popups allow-scripts allow-same-origin"
></iframe>
 * @export
 * @class PrefersColorSchemeObserver
 * @alias observer/PrefersColorSchemeObserver
 * @extends EventEmitter
 * @throws {Error} browser 에서 지원하지 못하는 경우 - throw new Error('PrefersColorScheme 지원되지 않는 브라우저입니다.');
 * @example
import { PrefersColorSchemeObserver, PREFERS_COLOR_SCHEME_EVENTS } from "@nonoll/code-snippet/observer";

const createElement = ({ tag = 'div', id = '', style = '', value = '', text = '' }) => {
  const doc = window.document;
  const target = doc.createElement(tag);
  target.setAttribute('id', id);
  target.setAttribute('style', style);
  target.setAttribute('value', value);
  if (text) {
    target.textContent = text;
  }
  return target;
}

let observer;

export const forExample = () => {
  if (observer) {
    console.log("already example");
    return;
  }

  const doc = window.document;

  observer = new PrefersColorSchemeObserver();

  observer.on(PREFERS_COLOR_SCHEME_EVENTS.CHANGE, ({ isDark, isLight }) => {
    console.log("change", isDark, isLight);
  });

  observer.attach();
  observer.emit(PREFERS_COLOR_SCHEME_EVENTS.GET_STATUS);

  const statusButton = createElement({ tag: "button", text: "status" });

  doc.body.appendChild(statusButton);

  statusButton.addEventListener("click", e => {
    e.preventDefault();
    console.log("statusButton clicked");
    if (!observer) {
      return;
    }
    observer.emit(PREFERS_COLOR_SCHEME_EVENTS.GET_STATUS);
  });
};

forExample();
 */
class PrefersColorSchemeObserver extends EventEmitter {
    constructor() {
        super();
        this.checkColorScheme = `(prefers-color-scheme: ${PREFERS_COLOR_SCHEME.DARK})`;
        /* istanbul ignore next: for not support error */
        if (!PrefersColorSchemeObserver.isSupport()) {
            throw new Error('PrefersColorScheme 지원되지 않는 브라우저입니다.');
        }
        this.onPrefersColorSchemeChangeListener = this.onPrefersColorSchemeChangeListener.bind(this);
    }
    /**
     * prefers-color-scheme 지원 여부 반환
     * @static
     * @returns {boolean}
     * @memberof PrefersColorSchemeObserver
     */
    static isSupport() {
        return !!window.matchMedia;
    }
    /**
     * 이벤트 감지 설정
     * @returns {PrefersColorSchemeObserver}
     * @memberof PrefersColorSchemeObserver
     */
    attach() {
        window.matchMedia(this.checkColorScheme).addEventListener('change', this.onPrefersColorSchemeChangeListener);
        this.on(PREFERS_COLOR_SCHEME_EVENTS.GET_STATUS, this.onPrefersColorSchemeChangeListener);
        return this;
    }
    /**
     * 이벤트 감지 해제
     * @returns {PrefersColorSchemeObserver}
     * @memberof PrefersColorSchemeObserver
     */
    detach() {
        window.matchMedia(this.checkColorScheme).removeEventListener('change', this.onPrefersColorSchemeChangeListener);
        this.off(PREFERS_COLOR_SCHEME_EVENTS.GET_STATUS, this.onPrefersColorSchemeChangeListener);
        return this;
    }
    /**
     * 현재 상태를 반환
     * @returns {IPrefersColorSchemeStatus}
     * @memberof PrefersColorSchemeObserver
     */
    getStatus() {
        const { matches } = window.matchMedia(this.checkColorScheme);
        const status = {
            isDark: matches,
            isLight: !matches
        };
        return status;
    }
    /**
     * 변화 감지 이벤트 리스너
     * @private
     * @memberof PrefersColorSchemeObserver
     * @fires PrefersColorSchemeObserver#PREFERS_COLOR_SCHEME_EVENTS
     */
    onPrefersColorSchemeChangeListener() {
        const status = this.getStatus();
        this.emit(PREFERS_COLOR_SCHEME_EVENTS.CHANGE, status);
    }
    /**
     * destroy
     * @returns {PrefersColorSchemeObserver}
     * @memberof PrefersColorSchemeObserver
     */
    destroy() {
        this.detach();
        return this;
    }
}

const doc = window.document;
/**
 * 실행된 browser 기준으로, visibilityChange event 에 맞는 vendorPrefix 반환
 * @memberof VisibilityChangeObserver
 * @export
 * @returns {String}
 */
function getVendorPrefix() {
    let prefix = '';
    if (typeof doc.hidden !== 'undefined') {
        prefix = '';
    } /* istanbul ignore next */
    else if (typeof doc.msHidden !== 'undefined') {
        prefix = 'ms';
    } /* istanbul ignore next */
    else if (typeof doc.webkitHidden !== 'undefined') {
        prefix = 'webkit';
    }
    return prefix;
}
/**
 * getVendorPrefix 의 값
 * @constant
 * @memberof VisibilityChangeObserver
 * @type {String}
 */
const VENDOR_PREFIX = getVendorPrefix();
/**
 * browser 상의 visibilitychange event 명
 * @constant
 * @memberof VisibilityChangeObserver
 * @type {String}
 */
const VISIBILITY_EVENT_NAME = `${VENDOR_PREFIX}visibilitychange`;
/**
 * browser 상의 visibility hidden 시점의 event 명
 * @constant
 * @memberof VisibilityChangeObserver
 * @type {String}
 */
const HIDDEN_METHOD_NAME = VENDOR_PREFIX ? /* istanbul ignore next */ `${VENDOR_PREFIX}Hidden` : 'hidden';
/**
 * VisibilityChange Event Types
 * @event VisibilityChangeObserver#VISIBILITY_EVENTS
 * @memberof VisibilityChangeObserver
 * @property {String} CHANGE - 변경 시점
 * @property {String} GET_STATUS - 현재 상태값 체크
 */
// for jsdoc
/**
 * @export
 * @readonly
 * @enum {VISIBILITY_EVENTS}
 */
var VISIBILITY_EVENTS;
(function (VISIBILITY_EVENTS) {
    VISIBILITY_EVENTS["CHANGE"] = "VISIBILITY_CHANGE-EVENTS-CHANGE";
    VISIBILITY_EVENTS["GET_STATUS"] = "VISIBILITY_CHANGE-EVENTS-GET_STATUS";
})(VISIBILITY_EVENTS || (VISIBILITY_EVENTS = {}));
/**
 * browser 의 visibility 변화를 감지
  <iframe
    src="https://codesandbox.io/embed/nonollcode-snippet-9gko8?autoresize=1&expanddevtools=1&fontsize=14&hidenavigation=1&initialpath=%2Fobserver-VisibilityChange.html&module=%2Fobserver-VisibilityChange.html&theme=dark"
    style="width:100%; height:500px; border:1px solid black; border-radius: 4px; overflow:hidden;"
    title="@nonoll/code-snippet"
    allow="geolocation; microphone; camera; midi; vr; accelerometer; gyroscope; payment; ambient-light-sensor; encrypted-media; usb"
    sandbox="allow-modals allow-forms allow-popups allow-scripts allow-same-origin"
  ></iframe>
 * @export
 * @class VisibilityChangeObserver
 * @alias observer/VisibilityChangeObserver
 * @extends {EventEmitter}
 * @see https://developer.mozilla.org/en-US/docs/Web/API/Page_Visibility_API
 * @see https://caniuse.com/#search=visibilityState
 * @throws {Error} browser 에서 지원하지 못하는 경우 - throw new Error('VisibilityChange 지원되지 않는 브라우저입니다.');
 * @example
import { VisibilityChangeObserver, VISIBILITY_EVENTS } from '@nonoll/code-snippet/observer';

const createElement = ({ tag = 'div', id = '', style = '', value = '', text = '' }) => {
  const doc = window.document;
  const target = doc.createElement(tag);
  target.setAttribute('id', id);
  target.setAttribute('style', style);
  target.setAttribute('value', value);
  if (text) {
    target.textContent = text;
  }
  return target;
}

let observer;

const forExample = () => {
  if (observer) {
    console.log('already example');
    return;
  }

  const doc = window.document;

  observer = new VisibilityChangeObserver();

  observer.on(VISIBILITY_EVENTS.CHANGE, ({ isHidden, isShow }) => {
    console.log('change', isHidden, isShow);
  });

  observer.attach();
  observer.emit(VISIBILITY_EVENTS.GET_STATUS);

  const statusButton = createElement({ tag: 'button', text: 'status' });
  const openButton = createElement({ tag: 'button', text: 'window open' });

  doc.body.appendChild(statusButton);
  doc.body.appendChild(openButton);

  statusButton.addEventListener('click', e => {
    e.preventDefault();
    console.log('statusButton clicked');
    if (!observer) {
      return;
    }
    // observer.emit(VISIBILITY_EVENTS.GET_STATUS);
    const { isHidden, isShow } = observer.getStatus();
    console.log('getStatus', isHidden, isShow);
  });

  openButton.addEventListener('click', e => {
    e.preventDefault();
    console.log('openButton clicked');
    if (!observer) {
      return;
    }
    const browser = window.open('https://nonoll.github.io/code-snippet/');
    if (!browser) {
      console.error('팝업이 차단되어 있습니다.');
    }
  });
}

forExample();
 */
class VisibilityChangeObserver extends EventEmitter {
    constructor() {
        super();
        /* istanbul ignore next: for not support error */
        if (!VisibilityChangeObserver.isSupport()) {
            throw new Error('VisibilityChange 지원되지 않는 브라우저입니다.');
        }
        this.onVisibilityChangeListener = this.onVisibilityChangeListener.bind(this);
    }
    /**
     * visibilitychange event 명 반환
     * @static
     * @returns {String}
     * @memberof VisibilityChangeObserver
     */
    static visibilityEventName() {
        return `${VENDOR_PREFIX}visibilitychange`;
    }
    /**
     * visibility hidden event 명 반환
     * @static
     * @returns {String}
     * @memberof VisibilityChangeObserver
     */
    static hiddenMethodName() {
        return HIDDEN_METHOD_NAME;
    }
    /**
     * visibility change 지원 여부 반환
     * @static
     * @returns {boolean}
     * @memberof VisibilityChangeObserver
     */
    static isSupport() {
        return typeof doc[VisibilityChangeObserver.hiddenMethodName()] !== 'undefined';
    }
    /**
     * @private
     * @memberof VisibilityChangeObserver
     * @see IVisibilityStatus
     * @fires VisibilityChangeObserver#VISIBILITY_EVENTS
     */
    onVisibilityChangeListener() {
        const isHidden = this.isHidden();
        this.emit(VISIBILITY_EVENTS.CHANGE, { isHidden, isShow: !isHidden });
    }
    /**
     * @private
     * @returns {Boolean}
     * @memberof VisibilityChangeObserver
     */
    isHidden() {
        return doc[VisibilityChangeObserver.hiddenMethodName()];
    }
    /**
     * 현재 상태를 반환
     * @returns {IVisibilityStatus}
     * @memberof VisibilityChangeObserver
     */
    getStatus() {
        this.emit(VISIBILITY_EVENTS.GET_STATUS);
        const isHidden = this.isHidden();
        return { isHidden, isShow: !isHidden };
    }
    /**
     * 이벤트 감지 설정
     * @returns {VisibilityChangeObserver}
     * @memberof VisibilityChangeObserver
     */
    attach() {
        doc.addEventListener(VisibilityChangeObserver.visibilityEventName(), this.onVisibilityChangeListener, false);
        this.on(VISIBILITY_EVENTS.GET_STATUS, this.onVisibilityChangeListener);
        return this;
    }
    /**
     * 이벤트 감지 해제
     * @returns {VisibilityChangeObserver}
     * @memberof VisibilityChangeObserver
     */
    detach() {
        doc.removeEventListener(VisibilityChangeObserver.visibilityEventName(), this.onVisibilityChangeListener, false);
        this.off(VISIBILITY_EVENTS.GET_STATUS, this.onVisibilityChangeListener);
        return this;
    }
    /**
     * destory
     * @returns {VisibilityChangeObserver}
     * @memberof VisibilityChangeObserver
     */
    destroy() {
        this.detach();
        return this;
    }
}

;
;
;
;
;
;

/**
 * @alias observer
 */

/*
 * Public API Surface of @nonoll/code-snippet/observer
 */

var publicApi_observer = /*#__PURE__*/Object.freeze({
    __proto__: null,
    INTERSECTION_EVENTS: INTERSECTION_EVENTS,
    get INTERSECTION_DIRECTIONS () { return INTERSECTION_DIRECTIONS; },
    ExtIntersectionObserverEntry: ExtIntersectionObserverEntry,
    IntersectionObserver: IntersectionObserver,
    ɵ0: ɵ0,
    get MutationRecordType () { return MutationRecordType; },
    get MUTATION_EVENTS () { return MUTATION_EVENTS; },
    MutationObserver: MutationObserver,
    get PREFERS_COLOR_SCHEME () { return PREFERS_COLOR_SCHEME; },
    get PREFERS_COLOR_SCHEME_EVENTS () { return PREFERS_COLOR_SCHEME_EVENTS; },
    PrefersColorSchemeObserver: PrefersColorSchemeObserver,
    getVendorPrefix: getVendorPrefix,
    VENDOR_PREFIX: VENDOR_PREFIX,
    VISIBILITY_EVENT_NAME: VISIBILITY_EVENT_NAME,
    HIDDEN_METHOD_NAME: HIDDEN_METHOD_NAME,
    get VISIBILITY_EVENTS () { return VISIBILITY_EVENTS; },
    VisibilityChangeObserver: VisibilityChangeObserver
});

/**
 * @module types
 */

/*
 * Public API Surface of @nonoll/code-snippet/types
 */

var publicApi_types = /*#__PURE__*/Object.freeze({
    __proto__: null
});

/*
 * Public API Surface of @nonoll/code-snippet
 */

/**
 * Generated bundle index. Do not edit.
 */

export { publicApi_browser as browser, publicApi_event as event, publicApi_functions as functions, publicApi_math as math, publicApi_observer as observer, publicApi_types as types };
//# sourceMappingURL=nonoll-code-snippet.js.map
