import { __assign, __read, __extends } from 'tslib';

/**
 * debounce
  <iframe
    src="https://codesandbox.io/embed/nonollcode-snippet-9gko8?autoresize=1&expanddevtools=1&fontsize=14&hidenavigation=1&initialpath=%2Ffunctions-debounce.html&module=%2Ffunctions-debounce.html&theme=dark"
    style="width:100%; height:500px; border:1px solid black; border-radius: 4px; overflow:hidden;"
    title="@nonoll/code-snippet"
    allow="geolocation; microphone; camera; midi; vr; accelerometer; gyroscope; payment; ambient-light-sensor; encrypted-media; usb"
    sandbox="allow-modals allow-forms allow-popups allow-scripts allow-same-origin"
  ></iframe>
 * @memberof module:functions
 * @function
 * @see https://css-tricks.com/debouncing-throttling-explained-examples/
 * @param {TypeVoidFunction} fn
 * @param {Number} [delay=0]
 * @returns {TypeVoidFunction}
 * @example
import { debounce } from '@nonoll/code-snippet/functions';

const forExample = () => {
  const delay = 1000 * 1;
  const listener = debounce(res => console.log(res), delay);

  listener('1');
  listener('2');
  listener('3');
  listener(`4 ${new Date()}`);

  setTimeout(() => {
    listener(`5 ${new Date()}`);
  }, delay);
}

forExample();
 */
var debounce = function (fn, /* istanbul ignore next */ delay) {
    if (delay === void 0) { delay = 0; }
    var timer;
    return function () {
        var args = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            args[_i] = arguments[_i];
        }
        window.clearTimeout(timer);
        timer = window.setTimeout(function () {
            fn.apply(null, args);
        }, delay);
    };
};

/**
 * deepFreeze
 * @param {any} target
 * @memberof module:functions
 * @function
 */
var deepFreeze = function (target) {
    if (Object.isFrozen(target)) {
        return target;
    }
    if (!Array.isArray(target) && Object.getPrototypeOf(target) !== Object.getPrototypeOf({})) {
        if (Object.getPrototypeOf(target) !== Object.getPrototypeOf(Function)) {
            throw new Error('not allow types');
        }
    }
    Object.freeze(target);
    Object.keys(target).forEach(function (key) { return deepFreeze(target[key]); });
    return target;
};

/**
 * empty function
 * @memberof module:functions
 * @alias noop
 */
function noop() { }

/**
 * throttle
  <iframe
    src="https://codesandbox.io/embed/nonollcode-snippet-9gko8?autoresize=1&expanddevtools=1&fontsize=14&hidenavigation=1&initialpath=%2Ffunctions-throttle.html&module=%2Ffunctions-throttle.html&theme=dark"
    style="width:100%; height:500px; border:1px solid black; border-radius: 4px; overflow:hidden;"
    title="@nonoll/code-snippet"
    allow="geolocation; microphone; camera; midi; vr; accelerometer; gyroscope; payment; ambient-light-sensor; encrypted-media; usb"
    sandbox="allow-modals allow-forms allow-popups allow-scripts allow-same-origin"
  ></iframe>
 * @memberof module:functions
 * @function
 * @see https://css-tricks.com/debouncing-throttling-explained-examples/
 * @param {TypeVoidFunction} fn
 * @param {Number} [delay=0]
 * @returns {TypeVoidFunction}
 * @example
import { debounce } from '@nonoll/code-snippet/functions';

const forExample = () => {
  const delay = 1000 * 1;
  const listener = throttle(res => console.log(res), delay);

  listener(`1 ${new Date()}`);
  listener('2');
  listener('3');
  listener('4');

  setTimeout(() => {
    listener(`5 ${new Date()}`);
  }, delay);
}

forExample();
 */
var throttle = function (fn, /* istanbul ignore next */ delay) {
    if (delay === void 0) { delay = 0; }
    var caller = function (args) {
        fn.apply(null, args);
        base = +new Date();
    };
    var base = null;
    var isLeading = true;
    var stamp = null;
    return function () {
        var args = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            args[_i] = arguments[_i];
        }
        if (isLeading) {
            caller(args);
            isLeading = false;
            return;
        }
        stamp = +new Date();
        /* istanbul ignore next */
        base = base || stamp;
        if ((stamp - base) >= delay) {
            /* istanbul ignore next */
            caller(args);
        }
    };
};

/**
 * @module module:functions
 */

/**
 * EventEmitter 의 기능과 동일한 맥락으로, 간단히 구성
  <iframe
    src="https://codesandbox.io/embed/nonollcode-snippet-9gko8?autoresize=1&expanddevtools=1&fontsize=14&hidenavigation=1&initialpath=%2Fevent-EventEmitter.html&module=%2Fevent-EventEmitter.html&theme=dark"
    style="width:100%; height:500px; border:1px solid black; border-radius: 4px; overflow:hidden;"
    title="@nonoll/code-snippet"
    allow="geolocation; microphone; camera; midi; vr; accelerometer; gyroscope; payment; ambient-light-sensor; encrypted-media; usb"
    sandbox="allow-modals allow-forms allow-popups allow-scripts allow-same-origin"
  ></iframe>
 * @export
 * @class EventEmitter
 * @alias event/EventEmitter
 * @memberof event
 * @see https://www.npmjs.com/package/events
 * @example
import { EventEmitter } from '@nonoll/code-snippet/event';

const createElement = ({ tag = 'div', id = '', style = '', value = '', text = '' }) => {
  const doc = window.document;
  const target = doc.createElement(tag);
  target.setAttribute('id', id);
  target.setAttribute('style', style);
  target.setAttribute('value', value);
  if (text) {
    target.textContent = text;
  }
  return target;
}

let ee;
const forExample = () => {
  const doc = window.document;

  const onButton = createElement({ tag: 'button', text: 'on button' });
  const offButton = createElement({ tag: 'button', text: 'off button' });
  const emitButton = createElement({ tag: 'button', text: 'emit button' });
  const EVENT_NAME = 'EVENT_EXAMPLE';

  doc.body.appendChild(onButton);
  doc.body.appendChild(offButton);
  doc.body.appendChild(emitButton);

  ee = new EventEmitter();
  ee.on(EVENT_NAME, res => console.log(res));

  onButton.addEventListener('click', e => {
    e.preventDefault();
    console.log('onButton clicked');
    if (!ee) {
      return;
    }
    ee.off(EVENT_NAME);
    ee.on(EVENT_NAME, res => console.log(res));
  });

  offButton.addEventListener('click', e => {
    e.preventDefault();
    console.log('offButton clicked');
    if (!ee) {
      return;
    }
    ee.off(EVENT_NAME);
  });

  emitButton.addEventListener('click', e => {
    e.preventDefault();
    console.log('emitButton clicked');
    if (!ee) {
      return;
    }
    ee.emit(EVENT_NAME, new Date());
  });
}

forExample();
 */
var EventEmitter = /** @class */ (function () {
    function EventEmitter() {
        this.events = [];
    }
    /**
     * 이벤트 감지 등록
     * @param {string} eventName
     * @param {TypeVoidFunction} [listener={@link noop}]
     * @param {*} [context]
     * @returns {EventEmitter}
     * @memberof EventEmitter
     */
    EventEmitter.prototype.on = function (eventName, /* istanbul ignore next: for noop */ listener, context) {
        if (listener === void 0) { listener = noop; }
        this.events.push({ eventName: eventName, listener: listener, context: context });
        return this;
    };
    /**
     * 이벤트 감지 해제
     * @param {string} eventName
     * @param {TypeVoidFunction} [listener]
     * @returns {EventEmitter}
     * @memberof EventEmitter
     */
    EventEmitter.prototype.off = function (eventName, listener) {
        // tslint:disable-next-line: max-line-length
        var matched = this.events.findIndex(function (regEvent) { return regEvent.eventName === eventName && (listener ? regEvent.listener === listener : true); });
        // tslint:disable-next-line: no-bitwise
        if (!~matched) {
            return false;
        }
        this.events.splice(matched, 1);
        return this;
    };
    /**
     * 이벤트 1회 감지 등록
     * @param {string} eventName
     * @param {TypeVoidFunction} [listener={@link noop}]
     * @param {*} [context]
     * @returns {EventEmitter}
     * @memberof EventEmitter
     */
    EventEmitter.prototype.once = function (eventName, /* istanbul ignore next: for noop */ listener, context) {
        var _this = this;
        if (listener === void 0) { listener = noop; }
        var onceWrapper = function () {
            var values = [];
            for (var _i = 0; _i < arguments.length; _i++) {
                values[_i] = arguments[_i];
            }
            _this.off(eventName, onceWrapper);
            listener.apply(context || listener, values);
        };
        this.on(eventName, onceWrapper, context);
        return this;
    };
    /**
     * 이벤트 전파
     * @param {string} eventName
     * @param {...any[]} values
     * @returns {EventEmitter}
     * @memberof EventEmitter
     */
    EventEmitter.prototype.emit = function (eventName) {
        var values = [];
        for (var _i = 1; _i < arguments.length; _i++) {
            values[_i - 1] = arguments[_i];
        }
        this.events.forEach(function (regEvent) {
            if (regEvent.eventName === eventName) {
                regEvent.listener.apply(regEvent.context || regEvent.listener, values);
            }
        });
        return this;
    };
    /**
     * 이벤트 전파
     * @param {string} eventName
     * @param {any} values
     * @returns {EventEmitter}
     * @memberof EventEmitter
     */
    EventEmitter.prototype.fire = function (eventName, values) {
        this.emit(eventName, values);
        return this;
    };
    return EventEmitter;
}());

;

/**
 * @alias event
 */

var ɵ0 = function (_, v) { return v / 100; };
var defaultThresholdList = new Array(101).fill(0).map(ɵ0);
/**
 * IntersectionEvent Types
 * @event IntersectionObserver#INTERSECTION_EVENTS
 * @memberof IntersectionObserver
 * @property {String} CHANGE - 변경 시점
 */
// for jsdoc
/**
 * @export
 * @readonly
 * @enum {INTERSECTION_EVENTS}
 */
var INTERSECTION_EVENTS = {
    CHANGE: 'INTERSECTION_OBSERVER-EVENTS-CHANGE'
};
/**
 * @typedef {String} INTERSECTION_DIRECTIONS
 * @alias INTERSECTION_DIRECTIONS
 * @memberof IntersectionObserver
 * @property {String} UP
 * @property {String} DOWN
 * @property {String} UNKNOWN
 */
// for jsdoc
/**
 * @export
 * @readonly
 * @enum {INTERSECTION_DIRECTIONS}
 */
var INTERSECTION_DIRECTIONS;
(function (INTERSECTION_DIRECTIONS) {
    INTERSECTION_DIRECTIONS["UP"] = "INTERSECTION_OBSERVER-DIRECTIONS-UP";
    INTERSECTION_DIRECTIONS["DOWN"] = "INTERSECTION_OBSERVER-DIRECTIONS-DOWN";
    INTERSECTION_DIRECTIONS["UNKNOWN"] = "INTERSECTION_OBSERVER-DIRECTIONS-UNKNOWN";
})(INTERSECTION_DIRECTIONS || (INTERSECTION_DIRECTIONS = {}));
;
var ExtIntersectionObserverEntry = /** @class */ (function () {
    function ExtIntersectionObserverEntry(initParams) {
        var target = initParams.target, _a = initParams.visibleRatio, visibleRatio = _a === void 0 ? 0 : _a, _b = initParams.options, options = _b === void 0 ? {} : _b;
        /* istanbul ignore next: for not support error */
        if (!target) {
            throw new Error('IntersectionObserver target 설정은 필수 값 입니다.');
        }
        var defParams = {
            root: null,
            rootMargin: '0px',
            threshold: defaultThresholdList
        };
        this.target = target;
        this.visibleRatio = visibleRatio;
        this.options = Object.assign({}, defParams, options);
    }
    ExtIntersectionObserverEntry.prototype.checkForIntersections = function (target, options) {
        var root = options.root, rootMargin = options.rootMargin;
        var rootMarginValues = this.parseRootMargin(rootMargin);
        var rootIsInDom = this.rootIsInDom(root);
        var rootRect = rootIsInDom ? this.getRootRect(root, rootMarginValues) : this.getEmptyRect();
        var boundingClientRect = this.getBoundingClientRect(target);
        var rootContainsTarget = this.rootContainsTarget(root, target);
        var intersectionRect = rootIsInDom && rootContainsTarget && this.computeTargetAndRootIntersection(target, rootRect, root);
        var isIntersecting = !!intersectionRect;
        var targetArea = boundingClientRect.width * boundingClientRect.height;
        var intersectionRatio = 0;
        if (targetArea) {
            var rect = intersectionRect || this.getEmptyRect();
            var intersectionArea = rect.width * rect.height;
            intersectionRatio = Number((intersectionArea / targetArea).toFixed(4));
        }
        else {
            intersectionRatio = isIntersecting ? 1 : 0;
        }
        return {
            boundingClientRect: boundingClientRect,
            intersectionRatio: intersectionRatio,
            intersectionRect: intersectionRect || this.getEmptyRect(),
            isIntersecting: isIntersecting,
            rootBounds: rootRect,
            target: target,
            time: window.performance && performance.now && performance.now()
        };
    };
    ExtIntersectionObserverEntry.prototype.computeTargetAndRootIntersection = function (target, rootRect, root) {
        if (target === void 0) { target = null; }
        if (rootRect === void 0) { rootRect = null; }
        if (root === void 0) { root = null; }
        if (window.getComputedStyle(target).display === 'none') {
            return false;
        }
        var targetRect = this.getBoundingClientRect(target);
        var intersectionRect = targetRect;
        var parent = this.getParentNode(target);
        var atRoot = false;
        while (!atRoot) {
            var parentRect = null;
            var parentComputedStyle = (parent.nodeType === 1 ? window.getComputedStyle(parent) : {});
            if (parentComputedStyle.display === 'none') {
                return;
            }
            if (parent === root) {
                atRoot = true;
                parentRect = rootRect;
            }
            else {
                if (parent !== document.body &&
                    parent !== document.documentElement &&
                    parentComputedStyle.overflow !== 'visible') {
                    parentRect = this.getBoundingClientRect(parent);
                }
            }
            if (parentRect) {
                intersectionRect = this.computeRectIntersection(parentRect, intersectionRect);
                if (!intersectionRect) {
                    break;
                }
            }
            parent = this.getParentNode(parent);
        }
        return intersectionRect;
    };
    ExtIntersectionObserverEntry.prototype.computeRectIntersection = function (rect1, rect2) {
        var top = Math.max(rect1.top, rect2.top);
        var bottom = Math.min(rect1.bottom, rect2.bottom);
        var left = Math.max(rect1.left, rect2.left);
        var right = Math.min(rect1.right, rect2.right);
        var width = right - left;
        var height = bottom - top;
        var values = { top: top, bottom: bottom, left: left, right: right, width: width, height: height };
        return (width >= 0 && height >= 0) && values;
    };
    ExtIntersectionObserverEntry.prototype.parseRootMargin = function (rootMargin) {
        if (rootMargin === void 0) { rootMargin = '0px'; }
        var margins = rootMargin.split(/\s+/).map(function (margin) {
            var parts = /^(-?\d*\.?\d+)(px|%)$/.exec(margin);
            if (!parts) {
                throw new Error('rootMargin must be specified in pixels or percent');
            }
            return { value: window.parseFloat(parts[1]), unit: parts[2] };
        });
        margins[1] = margins[1] || margins[0];
        margins[2] = margins[2] || margins[0];
        margins[3] = margins[3] || margins[1];
        return margins;
    };
    ExtIntersectionObserverEntry.prototype.rootIsInDom = function (root) {
        return root || this.containsDeep(window.document, root);
    };
    ExtIntersectionObserverEntry.prototype.rootContainsTarget = function (root, target) {
        return this.containsDeep(root || window.document, target);
    };
    ExtIntersectionObserverEntry.prototype.containsDeep = function (parent, child) {
        var node = child;
        while (node) {
            if (node === parent) {
                return true;
            }
            node = this.getParentNode(node);
        }
        return false;
    };
    ExtIntersectionObserverEntry.prototype.getParentNode = function (node) {
        var parent = node.parentNode;
        if (parent && parent.nodeType === 11 && parent.host) {
            // If the parent is a shadow root, return the host element.
            return parent.host;
        }
        return parent;
    };
    ExtIntersectionObserverEntry.prototype.getRootRect = function (root, rootMargin) {
        if (rootMargin === void 0) { rootMargin = []; }
        var rootRect;
        if (root) {
            rootRect = this.getBoundingClientRect(root);
        }
        else {
            // Use <html>/<body> instead of window since scroll bars affect size.
            var html = window.document.documentElement;
            var body = window.document.body;
            rootRect = {
                top: 0,
                left: 0,
                right: html.clientWidth || body.clientWidth,
                width: html.clientWidth || body.clientWidth,
                bottom: html.clientHeight || body.clientHeight,
                height: html.clientHeight || body.clientHeight
            };
        }
        return this.expandRectByRootMargin(rootRect, rootMargin);
    };
    ExtIntersectionObserverEntry.prototype.getBoundingClientRect = function (el) {
        var rect = null;
        try {
            rect = el.getBoundingClientRect();
        }
        catch (err) {
            // Ignore Windows 7 IE11 "Unspecified error"
            // https://github.com/w3c/IntersectionObserver/pull/205
        }
        if (!rect) {
            return this.getEmptyRect();
        }
        return rect;
    };
    ExtIntersectionObserverEntry.prototype.getEmptyRect = function () {
        var emptyRect = {
            bottom: 0,
            height: 0,
            left: 0,
            right: 0,
            top: 0,
            width: 0,
            x: 0,
            y: 0
        };
        return __assign(__assign({}, emptyRect), { toJSON: function () { return emptyRect; } });
    };
    ExtIntersectionObserverEntry.prototype.expandRectByRootMargin = function (rect, rootMargin) {
        if (rootMargin === void 0) { rootMargin = []; }
        var _a = __read(rootMargin.map(function (margin, i) {
            return margin.unit === 'px' ? margin.value : (margin.value * (i % 2 ? rect.width : rect.height)) / 100;
        }), 4), mt = _a[0], mr = _a[1], mb = _a[2], ml = _a[3];
        var top = rect.top, right = rect.right, bottom = rect.bottom, left = rect.left;
        var newRect = {
            top: top - mt,
            right: right + mr,
            bottom: bottom + mb,
            left: left - ml,
            x: 0,
            y: 0,
            width: 0,
            height: 0
        };
        newRect.x = newRect.left;
        newRect.y = newRect.top;
        newRect.width = newRect.right - newRect.left;
        newRect.height = newRect.bottom - newRect.top;
        return __assign(__assign({}, newRect), { toJSON: function () { return newRect; } });
    };
    ExtIntersectionObserverEntry.prototype.getChangeData = function () {
        var entry = this.checkForIntersections(this.target, this.options);
        var intersectionRatio = Math.floor(entry.intersectionRatio * 100);
        var isVisible = entry.intersectionRatio > this.visibleRatio;
        return {
            currentTarget: this.target,
            entry: entry,
            intersectionRatio: intersectionRatio,
            isVisible: isVisible,
            direction: INTERSECTION_DIRECTIONS.UNKNOWN
        };
    };
    return ExtIntersectionObserverEntry;
}());
var PureIntersectionObserver = /** @class */ (function () {
    function PureIntersectionObserver(callback, options) {
        // TODO polyfill
        // https://www.npmjs.com/package/intersection-observer
        // const IntersectionObserver = require('intersection-observer');
        // this.pure = new IntersectionObserver(callback);
        this.pure = new window.IntersectionObserver(callback, options);
    }
    PureIntersectionObserver.prototype.observe = function (target, options) {
        this.pure.observe(target, options);
    };
    PureIntersectionObserver.prototype.disconnect = function () {
        this.pure.disconnect();
    };
    return PureIntersectionObserver;
}());
/**
 * element 가 viewport 영역에 노출되는지 변화를 감지
 * @export
 * @class IntersectionObserver
 * @alias observer/IntersectionObserver
 * @extends {EventEmitter}
 * @see https://developer.mozilla.org/ko/docs/Web/API/IntersectionObserver/IntersectionObserver
 * @see https://caniuse.com/#search=IntersectionObserver
 * @throws {Error} target 옵션을 지정하지 않은 경우 - throw new Error('IntersectionObserver target 설정은 필수 값 입니다.');
 * @param {Partial<IIntersectionObserverExtOption>} initParams
 * @param {Element} initParams.target
 * @param {Number} [initParams.visibleRatio=0]
 * @param {Number} [initParams.throttle=300]
 * @param {Function|null} [initParams.callback={@link noop}] callback function
 * @param {IntersectionObserverInit} [initParams.options={}]
 * @example
 .. TODO
 */
var IntersectionObserver = /** @class */ (function (_super) {
    __extends(IntersectionObserver, _super);
    function IntersectionObserver(initParams) {
        var _this = _super.call(this) || this;
        _this.onIntersectionCallback = function (changeData) { };
        var target = initParams.target, _a = initParams.visibleRatio, visibleRatio = _a === void 0 ? 0 : _a, _b = initParams.throttle, throttle$1 = _b === void 0 ? 300 : _b, _c = initParams.callback, callback = _c === void 0 ? noop : _c, _d = initParams.options, options = _d === void 0 ? {} : _d;
        /* istanbul ignore next: for not support error */
        if (!target) {
            throw new Error('IntersectionObserver target 설정은 필수 값 입니다.');
        }
        var defParams = {
            root: null,
            rootMargin: '0px',
            threshold: defaultThresholdList
        };
        _this.target = target;
        _this.visibleRatio = visibleRatio;
        _this.prevRatio = 0;
        _this.onIntersectionCallback = callback;
        _this.onIntersectionListener = _this.onIntersectionListener.bind(_this);
        _this.options = Object.assign({}, defParams, options);
        if (throttle$1) {
            _this.observer = new PureIntersectionObserver(throttle(_this.onIntersectionListener, throttle$1), _this.options);
        }
        else {
            _this.observer = new PureIntersectionObserver(_this.onIntersectionListener, _this.options);
        }
        return _this;
    }
    /**
     * element 가 이미 노출된 경우 등, IntersectionObserver 등록으로 판단이 어려울 경우<br>
     * IntersectionObserver.getIntersectionEntry 를 이용하여 target 의 노출 여부를 판단
     * @static
     * @param {Partial<IIntersectionObserverExtOption>} initParams
     * @param {Element} initParams.target
     * @param {Number} [initParams.visibleRatio=0]
     * @param {Number} [initParams.throttle=300]
     * @param {Function|null} [initParams.callback={@link noop}] callback function
     * @param {IntersectionObserverInit} [initParams.options={}]
     * @returns {IIntersectionChangeData}
     * @memberof IntersectionObserver
     * @example
     .. TODO
     */
    IntersectionObserver.getIntersectionEntry = function (initParams) {
        return new ExtIntersectionObserverEntry(initParams).getChangeData();
    };
    /**
     * 변화 감지 이벤트 리스너
     * @private
     * @param {IntersectionObserverEntry[]} entries
     * [MDN IntersectionObserverEntry]{@link https://developer.mozilla.org/en-US/docs/Web/API/IntersectionObserverEntry}
     * @memberof IntersectionObserver
     * @fires IntersectionObserver#INTERSECTION_EVENTS
     */
    IntersectionObserver.prototype.onIntersectionListener = function (entries) {
        var entry = entries[0];
        var intersectionRatio = entry.intersectionRatio;
        var isVisible = intersectionRatio > this.visibleRatio;
        var direction = this.prevRatio > intersectionRatio ? INTERSECTION_DIRECTIONS.DOWN : INTERSECTION_DIRECTIONS.UP;
        var changeData = {
            currentTarget: this.target,
            entry: entry,
            intersectionRatio: Math.floor(intersectionRatio * 100),
            isVisible: isVisible,
            direction: direction
        };
        this.emit(INTERSECTION_EVENTS.CHANGE, changeData);
        this.onIntersectionCallback(changeData);
        this.prevRatio = intersectionRatio;
    };
    /**
     * 이벤트 감지 설정
     * @returns {IntersectionObserver}
     * @memberof IntersectionObserver
     */
    IntersectionObserver.prototype.attach = function () {
        this.observer.observe(this.target, this.options);
        return this;
    };
    /**
     * 이벤트 감지 해제
     * @returns {IntersectionObserver}
     * @memberof IntersectionObserver
     */
    IntersectionObserver.prototype.detach = function () {
        this.observer.disconnect();
        return this;
    };
    /**
     * destroy
     * @returns {IntersectionObserver}
     * @memberof IntersectionObserver
     */
    IntersectionObserver.prototype.destroy = function () {
        this.detach();
        this.observer = null;
        return this;
    };
    return IntersectionObserver;
}(EventEmitter));

/**
 * @typedef {String} MutationRecordType
 * @alias MutationRecordType
 * @memberof MutationObserver
 * @see https://developer.mozilla.org/ko/docs/Web/API/MutationObserver#MutationObserverInit
 * @property {String} childList
 * @property {String} attributes
 * @property {String} characterData
 * @property {String} subtree
 */
// for jsdoc
/**
 * @export
 * @readonly
 * @enum {MutationRecordType}
 */
var MutationRecordType;
(function (MutationRecordType) {
    MutationRecordType["childList"] = "childList";
    MutationRecordType["attributes"] = "attributes";
    MutationRecordType["characterData"] = "characterData";
    MutationRecordType["subtree"] = "subtree";
})(MutationRecordType || (MutationRecordType = {}));
/**
 * MutationEvent Types
 * @event MutationObserver#MUTATION_EVENTS
 * @memberof MutationObserver
 * @property {String} CHANGE_CHILD_LIST - childList 변경 시점
 * @property {String} CHANGE_SUBTREE - subtree 변경 시점
 * @property {String} CHANGE_ATTRIBUTES - attributes 변경 시점
 * @property {String} CHANGE_CHARACTER_DATA - data 변경 시점
 * @property {String} CHANGE - 기타 변경 시점
 * @property {String} WILD_CARD - 모든 변경 시점
 */
// for jsdoc
/**
 * @export
 * @readonly
 * @enum {MUTATION_EVENTS}
 */
var MUTATION_EVENTS;
(function (MUTATION_EVENTS) {
    MUTATION_EVENTS["CHANGE_CHILD_LIST"] = "MUTATION_OBERSERVER-EVENTS-CHANGE_CHILD_LIST";
    MUTATION_EVENTS["CHANGE_SUBTREE"] = "MUTATION_OBERSERVER-EVENTS-CHANGE_SUBTREE";
    MUTATION_EVENTS["CHANGE_ATTRIBUTES"] = "MUTATION_OBERSERVER-EVENTS-CHANGE_ATTRIBUTES";
    MUTATION_EVENTS["CHANGE_CHARACTER_DATA"] = "MUTATION_OBERSERVER-EVENTS-CHANGE_CHARACTER_DATA";
    MUTATION_EVENTS["CHANGE"] = "MUTATION_OBERSERVER-EVENTS-CHANGE";
    MUTATION_EVENTS["WILD_CARD"] = "MUTATION_OBERSERVER-EVENTS-CHANGE_WILCD_CARD";
})(MUTATION_EVENTS || (MUTATION_EVENTS = {}));
var PureMutationObserver = /** @class */ (function () {
    function PureMutationObserver(callback) {
        // TODO polyfill
        // const MutationObserver = require('mutation-observer');
        // this.pure = new MutationObserver(callback);
        this.pure = new window.MutationObserver(callback);
    }
    PureMutationObserver.prototype.observe = function (target, options) {
        this.pure.observe(target, options);
    };
    PureMutationObserver.prototype.disconnect = function () {
        this.pure.disconnect();
    };
    return PureMutationObserver;
}());
/**
 * target 으로 설정된 element 의 변화를 감지
  <iframe
    src="https://codesandbox.io/embed/nonollcode-snippet-9gko8?autoresize=1&expanddevtools=1&fontsize=14&hidenavigation=1&initialpath=%2Fobserver-MutationObserver.html&module=%2Fobserver-MutationObserver.html&theme=dark"
    style="width:100%; height:500px; border:1px solid black; border-radius: 4px; overflow:hidden;"
    title="@nonoll/code-snippet"
    allow="geolocation; microphone; camera; midi; vr; accelerometer; gyroscope; payment; ambient-light-sensor; encrypted-media; usb"
    sandbox="allow-modals allow-forms allow-popups allow-scripts allow-same-origin"
  ></iframe>
 * @export
 * @class MutationObserver
 * @alias observer/MutationObserver
 * @extends EventEmitter
 * @see https://developer.mozilla.org/ko/docs/Web/API/MutationObserver
 * @see https://github.com/webmodules/mutation-observer
 * @see https://developer.mozilla.org/ko/docs/Web/API/MutationObserver#MutationObserverInit
 * @param {Partial<IMutationObserverExtOption>} option
 * @param {HTMLElement} [option.target=null] target
 * @param {Number} [option.debounce=300] debounce
 * @param {Function|null} [option.callback={@link noop}] callback function
 * @param {Object} [option.options={}]
 * @param {Boolean} [option.options.childList]
 * @param {Boolean} [option.options.attributes]
 * @param {Boolean} [option.options.characterData]
 * @param {Boolean} [option.options.subtree]
 * @param {Boolean} [option.options.attributeOldValue]
 * @param {Boolean} [option.options.characterDataOldValue]
 * @param {Array.<String>} [option.options.attributeFilter] [MDN attributeFilter]{@link https://developer.mozilla.org/en-US/docs/Web/API/MutationObserverInit/attributeFilter}
 * @example
import { MutationObserver, MUTATION_EVENTS } from '@nonoll/code-snippet/observer';

const createElement = ({ tag = 'div', id = '', style = '', value = '', text = '' }) => {
  const doc = window.document;
  const target = doc.createElement(tag);
  target.setAttribute('id', id);
  target.setAttribute('style', style);
  target.setAttribute('value', value);
  if (text) {
    target.textContent = text;
  }
  return target;
}

let observer;

const forExample = () => {
  if (observer) {
    console.log('already example');
    return;
  }

  const doc = window.document;

  const target = createElement({ id: 'example_target', style: 'border: 1px solid red' });
  doc.body.appendChild(target);

  const options = {
    childList: true,
    subtree: true
  };

  observer = new MutationObserver({ target, options });

  observer.on(MUTATION_EVENTS.WILD_CARD, (type, values) => {
    console.log('wildCard', type, values);
  }).on(MUTATION_EVENTS.CHANGE_CHILD_LIST, values => {
    console.log('childList', values);
  });

  observer.attach();

  const attachButton = createElement({ tag: 'button', text: 'observer attach' });
  const detachButton = createElement({ tag: 'button', text: 'observer detach' });
  const appendButton = createElement({ tag: 'button', text: 'append' });

  doc.body.appendChild(attachButton);
  doc.body.appendChild(detachButton);
  doc.body.appendChild(appendButton);

  attachButton.addEventListener('click', e => {
    e.preventDefault();
    console.log('attachButton clicked');
    if (!observer) {
      return;
    }
    observer.attach();
  });

  detachButton.addEventListener('click', e => {
    e.preventDefault();
    console.log('detachButton clicked');
    if (!observer) {
      return;
    }
    observer.detach();
  });

  appendButton.addEventListener('click', e => {
    e.preventDefault();
    console.log('appendButton clicked');
    if (!observer) {
      return;
    }
    const input = createElement({ tag: 'input', value: `${+new Date()}` });
    target.appendChild(input);
  });
}

forExample();
 */
var MutationObserver = /** @class */ (function (_super) {
    __extends(MutationObserver, _super);
    function MutationObserver(_a) {
        var _b = _a.target, target = _b === void 0 ? null : _b, _c = _a.debounce, debounce$1 = _c === void 0 ? 300 : _c, _d = _a.callback, callback = _d === void 0 ? noop : _d, _e = _a.options, options = _e === void 0 ? {} : _e;
        var _this = _super.call(this) || this;
        var defaultOptions = { childList: true, subtree: true };
        _this.target = target;
        _this.options = Object.assign({}, defaultOptions, options);
        _this.onMutationObserverListener = _this.onMutationObserverListener.bind(_this);
        /* istanbul ignore next: for noop */
        _this.callback = callback || noop;
        if (debounce$1) {
            _this.observer = new PureMutationObserver(debounce(_this.onMutationObserverListener, debounce$1));
        }
        else {
            _this.observer = new PureMutationObserver(_this.onMutationObserverListener);
        }
        return _this;
    }
    /**
     * 이벤트 감지 설정
     * @returns {MutationObserver}
     * @memberof MutationObserver
     * @example
     observer.attach();
     */
    MutationObserver.prototype.attach = function () {
        this.observer.observe(this.target, this.options);
        return this;
    };
    /**
     * 이벤트 감지 등록
     * @override
     * @param {string} eventName
     * @param {TypeVoidFunction} [listener={@link noop}]
     * @param {*} [context]
     * @memberof MutationObserver
     * @listens MutationObserver#MUTATION_EVENTS
     * @returns {MutationObserver}
     */
    MutationObserver.prototype.on = function (eventName, /* istanbul ignore next: for noop */ listener, context) {
        if (listener === void 0) { listener = noop; }
        _super.prototype.on.call(this, eventName, listener, context);
        return this;
    };
    /**
     * 변화 감지 이벤트 리스너
     * @private
     * @param {MutationRecord[]} mutations [MDN MutationRecord]{@link https://developer.mozilla.org/en-US/docs/Web/API/MutationRecord}
     * @memberof MutationObserver
     * @fires MutationObserver#MUTATION_EVENTS
     */
    MutationObserver.prototype.onMutationObserverListener = function (mutations) {
        var _this = this;
        mutations.forEach(function (mutation) {
            var type = mutation.type;
            var values = { type: type, mutation: mutation };
            switch (type) {
                case MutationRecordType.childList:
                    _this.callback(values);
                    _this.emit(MUTATION_EVENTS.CHANGE_CHILD_LIST, values);
                    _this.emit(MUTATION_EVENTS.WILD_CARD, MUTATION_EVENTS.CHANGE_CHILD_LIST, values);
                    break;
                case MutationRecordType.subtree:
                    _this.callback(values);
                    _this.emit(MUTATION_EVENTS.CHANGE_SUBTREE, values);
                    _this.emit(MUTATION_EVENTS.WILD_CARD, MUTATION_EVENTS.CHANGE_SUBTREE, values);
                    break;
                case MutationRecordType.attributes:
                    _this.callback(values);
                    _this.emit(MUTATION_EVENTS.CHANGE_ATTRIBUTES, values);
                    _this.emit(MUTATION_EVENTS.WILD_CARD, MUTATION_EVENTS.CHANGE_ATTRIBUTES, values);
                    break;
                case MutationRecordType.characterData:
                    _this.callback(values);
                    _this.emit(MUTATION_EVENTS.CHANGE_CHARACTER_DATA, values);
                    _this.emit(MUTATION_EVENTS.WILD_CARD, MUTATION_EVENTS.CHANGE_CHARACTER_DATA, values);
                    break;
                /* istanbul ignore next: for default */
                default:
                    _this.callback(values);
                    _this.emit(MUTATION_EVENTS.CHANGE, type, values);
                    _this.emit(MUTATION_EVENTS.WILD_CARD, type, values);
                    break;
            }
        });
    };
    /**
     * 이벤트 감지 해제
     * @returns {MutationObserver}
     * @memberof MutationObserver
     * @example
     observer.detach();
     */
    MutationObserver.prototype.detach = function () {
        if (this.observer) {
            this.observer.disconnect();
        }
        return this;
    };
    /**
     * destory
     * @returns {MutationObserver}
     * @memberof MutationObserver
     * @example
     observer.destroy();
     */
    MutationObserver.prototype.destroy = function () {
        this.detach();
        this.observer = null;
        return this;
    };
    return MutationObserver;
}(EventEmitter));

/**
 * @typedef {String} PREFERS_COLOR_SCHEME
 * @alias PREFERS_COLOR_SCHEME
 * @memberof PrefersColorSchemeObserver
 * @property {String} LIGHT
 * @property {String} DARK
 */
// for jsdoc
/**
 * @export
 * @readonly
 * @enum {PREFERS_COLOR_SCHEME}
 */
var PREFERS_COLOR_SCHEME;
(function (PREFERS_COLOR_SCHEME) {
    PREFERS_COLOR_SCHEME["LIGHT"] = "light";
    PREFERS_COLOR_SCHEME["DARK"] = "dark";
})(PREFERS_COLOR_SCHEME || (PREFERS_COLOR_SCHEME = {}));
/**
 * PrefersColorSchemeObserverEvent Types
 * @event PrefersColorSchemeObserver#PREFERS_COLOR_SCHEME_EVENTS
 * @memberof PrefersColorSchemeObserver
 * @property {String} CHANGE - 변경 시점
 * @property {String} GET_STATUS - 현재 상태값 체크
 */
// for jsdoc
/**
 * @export
 * @readonly
 * @enum {PREFERS_COLOR_SCHEME_EVENTS}
 */
var PREFERS_COLOR_SCHEME_EVENTS;
(function (PREFERS_COLOR_SCHEME_EVENTS) {
    PREFERS_COLOR_SCHEME_EVENTS["CHANGE"] = "PREFERS_COLOR_SCHEME_OBSERVER-EVENTS-CHANGE";
    PREFERS_COLOR_SCHEME_EVENTS["GET_STATUS"] = "PREFERS_COLOR_SCHEME_OBSERVER-EVENTS-GET_COLOR_SCHEME";
})(PREFERS_COLOR_SCHEME_EVENTS || (PREFERS_COLOR_SCHEME_EVENTS = {}));
/**
 * prefers-color-scheme 상태 변화를 감지
<iframe
  src="https://codesandbox.io/embed/nonollcode-snippet-9gko8?autoresize=1&expanddevtools=1&fontsize=14&hidenavigation=1&initialpath=%2Fobsever-PrefersColorScheme.html&module=%2Fobsever-PrefersColorScheme.html&theme=dark"
  style="width:100%; height:500px; border:1px solid black; border-radius: 4px; overflow:hidden;"
  title="@nonoll/code-snippet"
  allow="geolocation; microphone; camera; midi; vr; accelerometer; gyroscope; payment; ambient-light-sensor; encrypted-media; usb"
  sandbox="allow-modals allow-forms allow-popups allow-scripts allow-same-origin"
></iframe>
 * @export
 * @class PrefersColorSchemeObserver
 * @alias observer/PrefersColorSchemeObserver
 * @extends EventEmitter
 * @throws {Error} browser 에서 지원하지 못하는 경우 - throw new Error('PrefersColorScheme 지원되지 않는 브라우저입니다.');
 * @example
import { PrefersColorSchemeObserver, PREFERS_COLOR_SCHEME_EVENTS } from "@nonoll/code-snippet/observer";

const createElement = ({ tag = 'div', id = '', style = '', value = '', text = '' }) => {
  const doc = window.document;
  const target = doc.createElement(tag);
  target.setAttribute('id', id);
  target.setAttribute('style', style);
  target.setAttribute('value', value);
  if (text) {
    target.textContent = text;
  }
  return target;
}

let observer;

export const forExample = () => {
  if (observer) {
    console.log("already example");
    return;
  }

  const doc = window.document;

  observer = new PrefersColorSchemeObserver();

  observer.on(PREFERS_COLOR_SCHEME_EVENTS.CHANGE, ({ isDark, isLight }) => {
    console.log("change", isDark, isLight);
  });

  observer.attach();
  observer.emit(PREFERS_COLOR_SCHEME_EVENTS.GET_STATUS);

  const statusButton = createElement({ tag: "button", text: "status" });

  doc.body.appendChild(statusButton);

  statusButton.addEventListener("click", e => {
    e.preventDefault();
    console.log("statusButton clicked");
    if (!observer) {
      return;
    }
    observer.emit(PREFERS_COLOR_SCHEME_EVENTS.GET_STATUS);
  });
};

forExample();
 */
var PrefersColorSchemeObserver = /** @class */ (function (_super) {
    __extends(PrefersColorSchemeObserver, _super);
    function PrefersColorSchemeObserver() {
        var _this = _super.call(this) || this;
        _this.checkColorScheme = "(prefers-color-scheme: " + PREFERS_COLOR_SCHEME.DARK + ")";
        /* istanbul ignore next: for not support error */
        if (!PrefersColorSchemeObserver.isSupport()) {
            throw new Error('PrefersColorScheme 지원되지 않는 브라우저입니다.');
        }
        _this.onPrefersColorSchemeChangeListener = _this.onPrefersColorSchemeChangeListener.bind(_this);
        return _this;
    }
    /**
     * prefers-color-scheme 지원 여부 반환
     * @static
     * @returns {boolean}
     * @memberof PrefersColorSchemeObserver
     */
    PrefersColorSchemeObserver.isSupport = function () {
        return !!window.matchMedia;
    };
    /**
     * 이벤트 감지 설정
     * @returns {PrefersColorSchemeObserver}
     * @memberof PrefersColorSchemeObserver
     */
    PrefersColorSchemeObserver.prototype.attach = function () {
        window.matchMedia(this.checkColorScheme).addEventListener('change', this.onPrefersColorSchemeChangeListener);
        this.on(PREFERS_COLOR_SCHEME_EVENTS.GET_STATUS, this.onPrefersColorSchemeChangeListener);
        return this;
    };
    /**
     * 이벤트 감지 해제
     * @returns {PrefersColorSchemeObserver}
     * @memberof PrefersColorSchemeObserver
     */
    PrefersColorSchemeObserver.prototype.detach = function () {
        window.matchMedia(this.checkColorScheme).removeEventListener('change', this.onPrefersColorSchemeChangeListener);
        this.off(PREFERS_COLOR_SCHEME_EVENTS.GET_STATUS, this.onPrefersColorSchemeChangeListener);
        return this;
    };
    /**
     * 현재 상태를 반환
     * @returns {IPrefersColorSchemeStatus}
     * @memberof PrefersColorSchemeObserver
     */
    PrefersColorSchemeObserver.prototype.getStatus = function () {
        var matches = window.matchMedia(this.checkColorScheme).matches;
        var status = {
            isDark: matches,
            isLight: !matches
        };
        return status;
    };
    /**
     * 변화 감지 이벤트 리스너
     * @private
     * @memberof PrefersColorSchemeObserver
     * @fires PrefersColorSchemeObserver#PREFERS_COLOR_SCHEME_EVENTS
     */
    PrefersColorSchemeObserver.prototype.onPrefersColorSchemeChangeListener = function () {
        var status = this.getStatus();
        this.emit(PREFERS_COLOR_SCHEME_EVENTS.CHANGE, status);
    };
    /**
     * destroy
     * @returns {PrefersColorSchemeObserver}
     * @memberof PrefersColorSchemeObserver
     */
    PrefersColorSchemeObserver.prototype.destroy = function () {
        this.detach();
        return this;
    };
    return PrefersColorSchemeObserver;
}(EventEmitter));

var doc = window.document;
/**
 * 실행된 browser 기준으로, visibilityChange event 에 맞는 vendorPrefix 반환
 * @memberof VisibilityChangeObserver
 * @export
 * @returns {String}
 */
function getVendorPrefix() {
    var prefix = '';
    if (typeof doc.hidden !== 'undefined') {
        prefix = '';
    } /* istanbul ignore next */
    else if (typeof doc.msHidden !== 'undefined') {
        prefix = 'ms';
    } /* istanbul ignore next */
    else if (typeof doc.webkitHidden !== 'undefined') {
        prefix = 'webkit';
    }
    return prefix;
}
/**
 * getVendorPrefix 의 값
 * @constant
 * @memberof VisibilityChangeObserver
 * @type {String}
 */
var VENDOR_PREFIX = getVendorPrefix();
/**
 * browser 상의 visibilitychange event 명
 * @constant
 * @memberof VisibilityChangeObserver
 * @type {String}
 */
var VISIBILITY_EVENT_NAME = VENDOR_PREFIX + "visibilitychange";
/**
 * browser 상의 visibility hidden 시점의 event 명
 * @constant
 * @memberof VisibilityChangeObserver
 * @type {String}
 */
var HIDDEN_METHOD_NAME = VENDOR_PREFIX ? /* istanbul ignore next */ VENDOR_PREFIX + "Hidden" : 'hidden';
/**
 * VisibilityChange Event Types
 * @event VisibilityChangeObserver#VISIBILITY_EVENTS
 * @memberof VisibilityChangeObserver
 * @property {String} CHANGE - 변경 시점
 * @property {String} GET_STATUS - 현재 상태값 체크
 */
// for jsdoc
/**
 * @export
 * @readonly
 * @enum {VISIBILITY_EVENTS}
 */
var VISIBILITY_EVENTS;
(function (VISIBILITY_EVENTS) {
    VISIBILITY_EVENTS["CHANGE"] = "VISIBILITY_CHANGE-EVENTS-CHANGE";
    VISIBILITY_EVENTS["GET_STATUS"] = "VISIBILITY_CHANGE-EVENTS-GET_STATUS";
})(VISIBILITY_EVENTS || (VISIBILITY_EVENTS = {}));
/**
 * browser 의 visibility 변화를 감지
  <iframe
    src="https://codesandbox.io/embed/nonollcode-snippet-9gko8?autoresize=1&expanddevtools=1&fontsize=14&hidenavigation=1&initialpath=%2Fobserver-VisibilityChange.html&module=%2Fobserver-VisibilityChange.html&theme=dark"
    style="width:100%; height:500px; border:1px solid black; border-radius: 4px; overflow:hidden;"
    title="@nonoll/code-snippet"
    allow="geolocation; microphone; camera; midi; vr; accelerometer; gyroscope; payment; ambient-light-sensor; encrypted-media; usb"
    sandbox="allow-modals allow-forms allow-popups allow-scripts allow-same-origin"
  ></iframe>
 * @export
 * @class VisibilityChangeObserver
 * @alias observer/VisibilityChangeObserver
 * @extends {EventEmitter}
 * @see https://developer.mozilla.org/en-US/docs/Web/API/Page_Visibility_API
 * @see https://caniuse.com/#search=visibilityState
 * @throws {Error} browser 에서 지원하지 못하는 경우 - throw new Error('VisibilityChange 지원되지 않는 브라우저입니다.');
 * @example
import { VisibilityChangeObserver, VISIBILITY_EVENTS } from '@nonoll/code-snippet/observer';

const createElement = ({ tag = 'div', id = '', style = '', value = '', text = '' }) => {
  const doc = window.document;
  const target = doc.createElement(tag);
  target.setAttribute('id', id);
  target.setAttribute('style', style);
  target.setAttribute('value', value);
  if (text) {
    target.textContent = text;
  }
  return target;
}

let observer;

const forExample = () => {
  if (observer) {
    console.log('already example');
    return;
  }

  const doc = window.document;

  observer = new VisibilityChangeObserver();

  observer.on(VISIBILITY_EVENTS.CHANGE, ({ isHidden, isShow }) => {
    console.log('change', isHidden, isShow);
  });

  observer.attach();
  observer.emit(VISIBILITY_EVENTS.GET_STATUS);

  const statusButton = createElement({ tag: 'button', text: 'status' });
  const openButton = createElement({ tag: 'button', text: 'window open' });

  doc.body.appendChild(statusButton);
  doc.body.appendChild(openButton);

  statusButton.addEventListener('click', e => {
    e.preventDefault();
    console.log('statusButton clicked');
    if (!observer) {
      return;
    }
    // observer.emit(VISIBILITY_EVENTS.GET_STATUS);
    const { isHidden, isShow } = observer.getStatus();
    console.log('getStatus', isHidden, isShow);
  });

  openButton.addEventListener('click', e => {
    e.preventDefault();
    console.log('openButton clicked');
    if (!observer) {
      return;
    }
    const browser = window.open('https://nonoll.github.io/code-snippet/');
    if (!browser) {
      console.error('팝업이 차단되어 있습니다.');
    }
  });
}

forExample();
 */
var VisibilityChangeObserver = /** @class */ (function (_super) {
    __extends(VisibilityChangeObserver, _super);
    function VisibilityChangeObserver() {
        var _this = _super.call(this) || this;
        /* istanbul ignore next: for not support error */
        if (!VisibilityChangeObserver.isSupport()) {
            throw new Error('VisibilityChange 지원되지 않는 브라우저입니다.');
        }
        _this.onVisibilityChangeListener = _this.onVisibilityChangeListener.bind(_this);
        return _this;
    }
    /**
     * visibilitychange event 명 반환
     * @static
     * @returns {String}
     * @memberof VisibilityChangeObserver
     */
    VisibilityChangeObserver.visibilityEventName = function () {
        return VENDOR_PREFIX + "visibilitychange";
    };
    /**
     * visibility hidden event 명 반환
     * @static
     * @returns {String}
     * @memberof VisibilityChangeObserver
     */
    VisibilityChangeObserver.hiddenMethodName = function () {
        return HIDDEN_METHOD_NAME;
    };
    /**
     * visibility change 지원 여부 반환
     * @static
     * @returns {boolean}
     * @memberof VisibilityChangeObserver
     */
    VisibilityChangeObserver.isSupport = function () {
        return typeof doc[VisibilityChangeObserver.hiddenMethodName()] !== 'undefined';
    };
    /**
     * @private
     * @memberof VisibilityChangeObserver
     * @see IVisibilityStatus
     * @fires VisibilityChangeObserver#VISIBILITY_EVENTS
     */
    VisibilityChangeObserver.prototype.onVisibilityChangeListener = function () {
        var isHidden = this.isHidden();
        this.emit(VISIBILITY_EVENTS.CHANGE, { isHidden: isHidden, isShow: !isHidden });
    };
    /**
     * @private
     * @returns {Boolean}
     * @memberof VisibilityChangeObserver
     */
    VisibilityChangeObserver.prototype.isHidden = function () {
        return doc[VisibilityChangeObserver.hiddenMethodName()];
    };
    /**
     * 현재 상태를 반환
     * @returns {IVisibilityStatus}
     * @memberof VisibilityChangeObserver
     */
    VisibilityChangeObserver.prototype.getStatus = function () {
        this.emit(VISIBILITY_EVENTS.GET_STATUS);
        var isHidden = this.isHidden();
        return { isHidden: isHidden, isShow: !isHidden };
    };
    /**
     * 이벤트 감지 설정
     * @returns {VisibilityChangeObserver}
     * @memberof VisibilityChangeObserver
     */
    VisibilityChangeObserver.prototype.attach = function () {
        doc.addEventListener(VisibilityChangeObserver.visibilityEventName(), this.onVisibilityChangeListener, false);
        this.on(VISIBILITY_EVENTS.GET_STATUS, this.onVisibilityChangeListener);
        return this;
    };
    /**
     * 이벤트 감지 해제
     * @returns {VisibilityChangeObserver}
     * @memberof VisibilityChangeObserver
     */
    VisibilityChangeObserver.prototype.detach = function () {
        doc.removeEventListener(VisibilityChangeObserver.visibilityEventName(), this.onVisibilityChangeListener, false);
        this.off(VISIBILITY_EVENTS.GET_STATUS, this.onVisibilityChangeListener);
        return this;
    };
    /**
     * destory
     * @returns {VisibilityChangeObserver}
     * @memberof VisibilityChangeObserver
     */
    VisibilityChangeObserver.prototype.destroy = function () {
        this.detach();
        return this;
    };
    return VisibilityChangeObserver;
}(EventEmitter));

;
;
;
;
;
;

/**
 * @alias observer
 */

/*
 * Public API Surface of @nonoll/code-snippet/observer
 */

/**
 * Generated bundle index. Do not edit.
 */

export { ExtIntersectionObserverEntry, HIDDEN_METHOD_NAME, INTERSECTION_DIRECTIONS, INTERSECTION_EVENTS, IntersectionObserver, MUTATION_EVENTS, MutationObserver, MutationRecordType, PREFERS_COLOR_SCHEME, PREFERS_COLOR_SCHEME_EVENTS, PrefersColorSchemeObserver, VENDOR_PREFIX, VISIBILITY_EVENTS, VISIBILITY_EVENT_NAME, VisibilityChangeObserver, getVendorPrefix, ɵ0 };
//# sourceMappingURL=nonoll-code-snippet-observer.js.map
